#include "BlackFish.hpp"

/******************************************************************************
 * BlackFish
 * Copyright (C) 1999-2001 Sren Meyer-Eppler, Jutta Jordans
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/

#undef min
#undef max
#pragma warning ( push, 1 )
#include <stdio.h>				// sprintf
#include <math.h>
#include <cassert>
#pragma warning ( pop )

#include "general/Utilities.hpp"
#include "IniFile/inifile.hpp"
#include "BlackFish/time.hpp"
#include "BlackFish/paint.hpp"
#include "BlackFish/cache.hpp"
#include "BlackFish/dxdraw.hpp"
#include "BlackFish/tileset.hpp"
#include "BlackFish/sprite.hpp"
#include "BlackFish/PixelConverter.hpp"


/**
 * constructor, loads all map and map graphic resources
 */
CBlackFish::CBlackFish(
    CSurface *RenderTarget,
    CCache *Cache,
    const char *IniFileName )
	: RenderTarget( RenderTarget )
	 ,ScrapSurface( )
	 ,TransparentColor( 0 )
	 ,Cache( Cache )
	 ,NrOfTileSetsGround( 0 )
	 ,NrOfTileSetsOverlay( 0 )
	 ,NrOfTileSetsWall( 0 )
	 ,NrOfTileSetsSprite( 0 )
	 ,NrOfTileSets( 0 ) // should be the sum of the previous 4
	 ,Tiles( NULL )
	 ,WorldWidth( 0 )
	 ,WorldHeight( 0 )
	 ,WorldPos( )
	 ,Map( NULL )
	 ,ScrollSpeed( 0 )
	 ,ScrollTime( 0 )
     ,Scrolling( false )
	 ,ScrollDir( )
{
    assert( DirectDraw != NULL );
    assert( IniFileName != NULL );
    assert( RenderTarget != NULL );
    assert( Cache != NULL );

    CIniFile IniFile( IniFileName );

    bool ReturnValue = false;
    ReturnValue = IniFile.ReadNumber( "[BlackFish]", "NrOfTileSetsGround",	&NrOfTileSetsGround );  assert( ReturnValue );
    ReturnValue = IniFile.ReadNumber( "[BlackFish]", "NrOfTileSetsOverlay", &NrOfTileSetsOverlay ); assert( ReturnValue );
    ReturnValue = IniFile.ReadNumber( "[BlackFish]", "NrOfTileSetsWall",	&NrOfTileSetsWall );	assert( ReturnValue );
    ReturnValue = IniFile.ReadNumber( "[BlackFish]", "NrOfTileSetsSprite",	&NrOfTileSetsSprite );  assert( ReturnValue );
    ReturnValue = IniFile.ReadNumber( "[BlackFish]", "NrOfTileSetsMax",		&NrOfTileSets );		assert( ReturnValue );
    ReturnValue = IniFile.ReadNumber( "[BlackFish]", "ScrollSpeed",			&ScrollSpeed );			assert( ReturnValue );
	if ( ScrollSpeed > 100 ) ScrollSpeed = 1000;	// clamp value to range 0..1000

    char NameString[80];
    Tiles = NULL;
    Tiles = new CTileSet*[ static_cast<tUINT32>( NrOfTileSets )];
    assert( Tiles != NULL );

	tSINT32 i		= 0;
	tSINT32 Offset	= 0;

	for ( i=0; i<NrOfTileSets; i++ )
		Tiles[i] = NULL;
	for ( i=0; i<NrOfTileSetsGround; i++ )
	{
		sprintf( NameString, "[TileSetGround%i]", i);
        Tiles[i] = new CTileSet( IniFileName, NameString );
        assert( Tiles[i] != NULL );
        Tiles[i]->SetCache( Cache );
	}

	Offset += NrOfTileSets / 4;
	for ( i=Offset; i<NrOfTileSetsOverlay+Offset; i++ )
	{
		sprintf( NameString, "[TileSetOverlay%i]", i-Offset );
        Tiles[i] = new CTileSet( IniFileName, NameString );
        assert( Tiles[i] != NULL );
        Tiles[i]->SetCache( Cache );
	}

	Offset += NrOfTileSets / 4;
	for ( i=Offset; i<NrOfTileSetsWall+Offset; i++ )
	{
		sprintf( NameString, "[TileSetWall%i]", i-Offset );
        Tiles[i] = new CTileSet( IniFileName, NameString );
        assert( Tiles[i] != NULL );
        Tiles[i]->SetCache( Cache );
	}

	Offset += NrOfTileSets / 4;
	for ( i=Offset; i<NrOfTileSetsSprite+Offset; i++ )
	{
		sprintf( NameString, "[TileSetSprite%i]", i-Offset );
        Tiles[i] = new CTileSet( IniFileName, NameString );
        assert( Tiles[i] != NULL );
        Tiles[i]->SetCache( Cache );
	}

	// !! ugly, first two wall tiles are special (used for dither effect)
	Tiles[2 * NrOfTileSets / 4]->GetTile( 0 )->SetTransparentColor( 0 );
	Tiles[2 * NrOfTileSets / 4]->GetTile( 1 )->SetTransparentColor( 0 );

    CPixelConverter PixelConverter( RenderTarget );
    TransparentColor = PixelConverter.RGBToColor( 255, 0, 255 );	// remember pink transparent color
    ScrapSurface.Create( 64, 128, true );
	ScrapSurface.SetTransparentColor( TransparentColor );
	SetRect( &m_ViewRect, 100, 4, 700, 484 ); // !! ignore return value, what does it mean anyways? couldn't find in MSDN
	Map = NULL;

	// hack... read current mapfile from inifile, should later be replaced by
	// level initialisation method
    char MapFileName[80] = "";
    ReturnValue = IniFile.ReadString( "[BlackFish]", "MapFileName", MapFileName );
    assert( ReturnValue );
	ChangeMap( MapFileName );
}


/**
 * deletes all tilesets and the map object
 */
CBlackFish::~CBlackFish( )
{
	// free memory used by graphic tile sets
    for ( tSINT32 i=0; i<NrOfTileSets; i++)
        SAFEDELETE( Tiles[i] );
    SAFEDELETEARRAY( Tiles );
    SAFEDELETE( Map );

	RenderTarget= NULL;
	Cache		= NULL;
}


/**
 * switch current map to map 'FileName'
 */
void
CBlackFish::ChangeMap( const char *FileName )
{
	// start at map origin
	WorldPos.Set( 0.0f, 0.0f, 0.0f );

	// don't start off already scrolling
	ScrollTime		= 0;
    Scrolling       = false;
	ScrollDir.Set( 0.0f, 0.0f, 0.0f );

	// destroy old map
	SAFEDELETE( Map );
	// create the new map
    Map = new CMap( FileName );

	// adjust worldwidth and height to the new map dimensions
	WorldWidth	= 32 * Map->GetWidth();
	WorldHeight	= 32 * Map->GetHeight();
}


/**
 * start over with a new map initialized with the values in MapHeader and base layer tileset 0 tilenr 0
 */
void
CBlackFish::NewMap( const CMap::tMAPHEADER & MapHeader )
{
	assert( MapHeader.Width != 0 );
	assert( MapHeader.Width <= 1024 );	// !! maximum map dimension = 1024x1024
	assert( MapHeader.Height != 0 );		// !! arbitrary defined here good?
	assert( MapHeader.Height <= 1024 );

	// start at map origin
	WorldPos.Set( 0.0f, 0.0f, 0.0f );

	// don't start off already scrolling
	ScrollTime		= 0;
    Scrolling       = false;
	ScrollDir.Set( 0.0f, 0.0f, 0.0f );

	// destroy old map
	SAFEDELETE( Map );
	// create the new map
    Map = new CMap( MapHeader );

	// adjust worldwidth and height to the new map dimensions
	WorldWidth	= 32 * Map->GetWidth();
	WorldHeight	= 32 * Map->GetHeight();
}


/**
 * !!
 * Due to the Wobbler Problem the algorithm here isn't correct:
 * - Scroll speed is not equal along the world axes and the diagonals
 * - minimum speed is 1, which means you scroll at least ScrollSpeed
 *   units per second, but with very high framerates it might be more
 *   than that ( you scroll at least 1 pixel per frame )
 * - worldcoordinates are effectively truncated before use ( really
 *   the heart of the problem )
 * !! make both scroll functions inline?
 * !! we can't have this function called from multiple places at the moment
 *    i.e. keyboard and mouse scrolling don't work together because we don't
 *    have a begin and end scrolling method.
 */
void
CBlackFish::Scroll( tDIRECTION Direction )
{
	switch ( Direction )
	{
		case NORTHWEST:	ScrollDir.Set(  0.0f, -2.0f,  0.0f ); break;
		case NORTH:		ScrollDir.Set(  2.0f, -2.0f,  0.0f ); break;
		case NORTHEAST:	ScrollDir.Set(  2.0f,  0.0f,  0.0f ); break;
		case EAST:		ScrollDir.Set(  2.0f,  2.0f,  0.0f ); break;
		case SOUTHEAST:	ScrollDir.Set(  0.0f,  2.0f,  0.0f ); break;
		case SOUTH: 	ScrollDir.Set( -2.0f,  2.0f,  0.0f ); break;
		case SOUTHWEST:	ScrollDir.Set( -2.0f,  0.0f,  0.0f ); break;
		case WEST:		ScrollDir.Set( -2.0f, -2.0f,  0.0f ); break;
		case NONE:
		default:
			ScrollDir.Set(  0.0f,  0.0f,  0.0f );
            Scrolling = false;
		break;
	}

	tFLOAT Speed = 0;
	if ( Scrolling )	// we are already scrolling, update timer
	{
		Speed = static_cast<tFLOAT>(( Timer.GetTime() - ScrollTime ) * ScrollSpeed );
		Speed /= Timer.GetFrequency();
		// the next assumes a fixed framerate of 60Hz
//		Speed = Timer.GetFrequency() * ScrollSpeed;
//		Speed /= 60;
//		Speed /= Timer.GetFrequency();
	}

	if ( Direction != NONE )	// we are starting to scroll, initialize timer, don't scroll, yet
	{
		ScrollDir.Multiply( static_cast<tFLOAT> ( ceil ( static_cast<double>( Speed ))));
		WorldPos.Add( ScrollDir );
        Scrolling = true;

		WorldPos.x = ClipToRange( WorldPos.x, 0.0f, tFLOAT( WorldWidth ));
		WorldPos.y = ClipToRange( WorldPos.y, 0.0f, tFLOAT( WorldHeight ));
	}

	ScrollTime = Timer.GetTime( );
}


/**
 * shortcut function, allows the translation from mouse-coordinates into scrolling directions
 * should actually be done in game.cpp
 * !! inline this?
 */
void
CBlackFish::Scroll( tSINT32 PosX, tSINT32 PosY )
{
	if (( PosX == 0 ) && ( PosY == 0 ))
		Scroll( NORTHWEST );
	else if (( PosX == 0 ) && ( PosY == 599 ))
		Scroll( SOUTHWEST );
	else if (( PosX == 799 ) && ( PosY == 599 ))
		Scroll( SOUTHEAST );
	else if (( PosX == 799 ) && ( PosY == 0 ))
		Scroll( NORTHEAST );
	else if ( PosX == 0 )
		Scroll( WEST );
	else if ( PosX == 799 )
		Scroll( EAST );
	else if ( PosY == 0 )
		Scroll( NORTH );
	else if ( PosY == 599 )
		Scroll( SOUTH );
	else
		Scroll( NONE ); // !! problem
}


/**
 * Input: Screen Coordinates
 * OutPut: ArrayIndices of the Cell displayed at these coordinates
 * returns: false if ArrayIndices are out of bounds
 */
bool
CBlackFish::GetCellForScreen( tSINT32 & ArrayX, tSINT32 & ArrayY, tSINT32 ScreenX, tSINT32 ScreenY ) const
{
	assert( Map != NULL );

    // return false if pixel-position is not in view-rect
	if  (( ScreenX < m_ViewRect.left )
	  || ( ScreenX >= m_ViewRect.right )
	  || ( ScreenY < m_ViewRect.top )
	  || ( ScreenY >= m_ViewRect.bottom ))
      return false;

	tSINT32 PixelDiffX = (( m_ViewRect.right + m_ViewRect.left ) / 2 ) - ScreenX - 12;
	tSINT32 PixelDiffY = (( m_ViewRect.bottom + m_ViewRect.top ) / 2 ) - ScreenY;

	tSINT32 WorldDiffX = PixelDiffX / 2 - PixelDiffY;
	tSINT32 WorldDiffY = PixelDiffX / 2 + PixelDiffY;

	// this is here so for example world(-3,-3) and world(3,3) won't map to the same 
	// cell(0,0), but rather to (-1,-1) and (0,0) respectively
	tSINT32 ZeroX = ( static_cast<tSINT32>( WorldPos.x ) - WorldDiffX );
	if ( ZeroX < 0 ) ZeroX -= 32;
	tSINT32 ZeroY = ( static_cast<tSINT32>( WorldPos.y ) - WorldDiffY );
	if ( ZeroY < 0 ) ZeroY -= 32;

	ArrayX =  ZeroX / 32;
	ArrayY =  ZeroY / 32;

	if (( ArrayX < 0 ) || ( ArrayX >= Map->GetWidth( )))
        return false;
	if (( ArrayY < 0 ) || ( ArrayY >= Map->GetHeight( )))
        return false;
    return true;
}


/**
 * Where would a given cell (ArrayIndices) be displayed on-screen?
 * Note: Doesn't perform any out-of-bounds checks
 */
void
CBlackFish::GetBlitRectForCell( RECT & Rect, tSINT32 ArrayX, tSINT32 ArrayY ) const
{
	tSINT32 WorldDiffX = static_cast<tSINT32>( WorldPos.x ) - ( ArrayX * 32 );
	tSINT32 WorldDiffY = static_cast<tSINT32>( WorldPos.y ) - ( ArrayY * 32 );

	tSINT32 PixelDiffX = WorldDiffX + WorldDiffY;
	tSINT32 PixelDiffY = ( WorldDiffY - WorldDiffX ) / 2;

	Rect.left		= (( m_ViewRect.right + m_ViewRect.left ) / 2 ) - PixelDiffX - 12;
	Rect.right		= Rect.left + 64;
	Rect.top		= (( m_ViewRect.bottom + m_ViewRect.top ) / 2 ) - PixelDiffY - 16;
	Rect.bottom		= Rect.top + 32;
}


void
CBlackFish::GetScreenForWorld( tSINT32 & ScreenX, tSINT32 & ScreenY, const CVector & World ) const
{
	tSINT32 WorldDiffX = static_cast<tSINT32>( WorldPos.x - World.x );
	tSINT32 WorldDiffY = static_cast<tSINT32>( WorldPos.y - World.y );

	tSINT32 PixelDiffX = WorldDiffX + WorldDiffY;
	tSINT32 PixelDiffY = ( WorldDiffY - WorldDiffX ) / 2;

	ScreenX = (( m_ViewRect.right + m_ViewRect.left ) / 2 ) - PixelDiffX - 12;
	ScreenY = (( m_ViewRect.bottom + m_ViewRect.top ) / 2 ) - PixelDiffY - static_cast<tSINT32>( World.z ) - 16;
}


/**
 * return the number of tilesets of the specified type
 * MAX: maximum number of tilesets allowed (default 1024)
 * ALL: number of tilesets allocated
 * GROUND, OVERLAY, WALL, SPRITE: number of allocated tilesets of the respective type
 */
tSINT32
CBlackFish::GetNrOfTileSets( tTILESETTYPE Type ) const
{
	switch ( Type )
	{
	case MAX:
		return NrOfTileSets;
	case ALL:
		return NrOfTileSetsGround + NrOfTileSetsOverlay + NrOfTileSetsWall + NrOfTileSetsSprite;
	case GROUND:
		return NrOfTileSetsGround;
	case OVERLAY:
		return NrOfTileSetsOverlay;
	case WALL:
		return NrOfTileSetsWall;
	case SPRITE:
		return NrOfTileSetsSprite;
	default:
		return 0;
	}
}


/**
 * Renders GroundCells
 * Note: Possible to optimize clipping, since the clipped rectangle results
 *       don't change for the viewrect sides
 */
void
CBlackFish::RenderGround(
	tSINT32 ArrayStartX,
	tSINT32 ArrayStartY,
	tSINT32 PixelStartX,
	tSINT32 PixelStartY ) const
{
	assert( Map != NULL );

    RECT Dest, Source;
    CCellinfo *Layer  = NULL;
	tSINT32 ArrayPosX = ArrayStartX;
	tSINT32 ArrayPosY = ArrayStartY;
	tSINT32 PixelPosX = PixelStartX;
	tSINT32 PixelPosY = PixelStartY;

    for ( tSINT32 y = 0; y < TILESVISIBLEY; y++ )
    {
        for ( tSINT32 x = 0; x < TILESVISIBLEX; x++ )
        {
            // don't draw tiles outside of the map array bounds
			SetRect( &Source, 0, 0, 64, 32 );
			SetRect( &Dest, PixelPosX, PixelPosY, PixelPosX + 64, PixelPosY + 32 );

			if ( Clip( m_ViewRect, Source, Dest ))
			{
				if (( ArrayPosX >= 0 ) && ( ArrayPosX < Map->GetWidth()) &&
					( ArrayPosY >= 0 ) && ( ArrayPosY < Map->GetHeight()))
				{
					Layer = Map->GetFirstLayer( ArrayPosX, ArrayPosY );
					while ( Layer != NULL )
					{
						Tiles[Layer->TileSet]->BlitTrans( Layer->TileNr, &Source, *RenderTarget, &Dest );
						Layer = Layer->NextLayer;
					}
				}
				else
				{	// out of map array -> blit black tile
					Tiles[0]->BlitTrans( 1, &Source, *RenderTarget, &Dest );
				}
			}

			PixelPosX+=64;
            ArrayPosX++;
            ArrayPosY++;
        }

		PixelPosY += 16;
		PixelPosX = PixelStartX;

		if ( (y % 2) == 1 )
			ArrayStartX--;
		else
		{
			PixelPosX += 32;
			ArrayStartY++;
		}

		ArrayPosX = ArrayStartX;
		ArrayPosY = ArrayStartY;
	}
}


/**
 * !! this function is being called twice per frame on the same input (except for the dither flag)
 *    thus we could keep lots of the calculated info for the second time around. We could even
 *    go further and also exploit frame to frame coherence and keep a list of currently visible
 *    sprites instead of determining that set anew each frame.
 */
void
CBlackFish::RenderSprite( const std::list<CSprite *> & SpriteList, bool Dithered ) const
{
    std::list<CSprite *>::const_iterator Iterator;

	for ( Iterator = SpriteList.begin( ); Iterator != SpriteList.end( ); Iterator++ )
	{
		tSINT32 TileSet = (*Iterator)->GetTileSet( );
		tSINT32 TileNr	= (*Iterator)->GetTileNr( );

		tSINT32 ScreenX, ScreenY;
        CVector Position;
		(*Iterator)->GetPosition( Position );
		GetScreenForWorld( ScreenX, ScreenY, Position );

		tSINT32 Width	= Tiles[TileSet]->GetTileWidth( );
		tSINT32 Height	= Tiles[TileSet]->GetTileHeight( );
		if ( Height <= 32 ) 
			ScreenY += Height; 
		else 
			ScreenY += 32; 
	
		RECT Dest, Source;
		SetRect( &Source, 0, 0, Width, Height );
		SetRect( &Dest, ScreenX, ScreenY - Height, ScreenX + Width, ScreenY );

		if ( Clip( m_ViewRect, Source, Dest ))
			if ( ! Dithered )
				Tiles[TileSet]->BlitTrans( TileNr, &Source, *RenderTarget, &Dest );
			else
			{
                CSurface * DitherWall = Tiles[2 * NrOfTileSets / 4]->GetTile( Dest.left & 1 ^ Dest.top & 1 );
				Tiles[TileSet]->Blit( 
					TileNr, 
					&Source, 
					const_cast<CSurface &>( ScrapSurface ), // no bad, because ScrapSurface is a cache
					&Source );
				ScrapSurface.BlitTrans( *DitherWall, &Source, &Source );
				RenderTarget->BlitTrans( ScrapSurface, &Source, &Dest );
			}
	}
}


/**
 * Renders Walls and Sprites, algorithmically very similar to RenderGround()
 * so changes to either function should be considered for the other as well.
 * the reason for the seperation is drawing order, since we don't have any
 * per-pixel depth information ( = z-buffer, not practical for our purposes )
 * we have to use the painters algorithm:
 * - draw all the groundlayers, as these never block view on any other items
 *   (this has already been done at this point ( RenderGround()))
 * - Render all sprites for a particular cell, then render the wall for that cell
 * !! note: under some circumstances sprites may pop through north-west walls
 *    this hopefully won't be visible once we've implemented transparency
 *	  for the walls (all walls in front of the 'poppers' will be transparent
 *	  anyway). Other than that I can't think of a solution that'll work
 *    for all cases. (transparency apparently really fixes this, though
 *    a similar problem persists for other sprites...)
 */
void
CBlackFish::RenderWallsAndSprites(
	tSINT32 ArrayStartX,
	tSINT32 ArrayStartY,
	tSINT32 PixelStartX,
	tSINT32 PixelStartY,
    bool SmallWalls ) const
{
	assert( Map != NULL );

	tSINT32 ArrayPosX = ArrayStartX;
	tSINT32 ArrayPosY = ArrayStartY;
	tSINT32 PixelPosX = PixelStartX;
	tSINT32 PixelPosY = PixelStartY;

	// should probably be moved into the class body, since this function is called so often (performance)
    std::vector< const std::list<CSprite *> * > SpriteList;
	// !! magic number, reserve at least n slots for sprites, so reallocation occurs less frequently
	SpriteList.reserve( 64 ); 

    // plus x because walls at the bottom of the screen would be clipped out of existance too early otherwise...
    // (Siedler3-like bug ;) )
    for ( tSINT32 y = 0; y < TILESVISIBLEY + 6; y++ )
    {
        for ( tSINT32 x = 0; x < TILESVISIBLEX + 2; x++ )
        {
            if (( ArrayPosX >= 0 ) && ( ArrayPosX < Map->GetWidth( ))
			  &&( ArrayPosY >= 0 ) && ( ArrayPosY < Map->GetHeight( )))
			{
                const std::list<CSprite *> *CellSpriteList = Map->GetSpriteList( ArrayPosX, ArrayPosY );
				if ( !CellSpriteList->empty())
				{
					RenderSprite( *CellSpriteList );
					SpriteList.push_back( CellSpriteList );
				}
                CWallinfo *Wall = Map->GetWall( ArrayPosX, ArrayPosY );
				if ( Wall != NULL )
				{
				    RECT Dest, Source;
					if ( SmallWalls )
					{
						SetRect( &Source, 0, 80, 64, 128 );
						SetRect( &Dest, PixelPosX, PixelPosY - 16, PixelPosX + 64, PixelPosY + 32 );
					}
					else
					{
						SetRect( &Source, 0, 0, 64, 128 );
						SetRect( &Dest, PixelPosX, PixelPosY - 96, PixelPosX + 64, PixelPosY + 32 );
					}
					if ( Clip( m_ViewRect, Source, Dest ) )
						Tiles[Wall->TileSet]->BlitTrans( Wall->TileNr, &Source, *RenderTarget, &Dest );
				}
			}

			PixelPosX+=64;
			ArrayPosX++;
			ArrayPosY++;
        }

		PixelPosY += 16;
		PixelPosX = PixelStartX;

		if ( (y % 2) == 1 )
			ArrayStartX--;
		else
		{
			PixelPosX += 32;
			ArrayStartY++;
		}

		ArrayPosX = ArrayStartX;
		ArrayPosY = ArrayStartY;
	}

    for ( std::vector< const std::list<CSprite *> * >::const_iterator i = SpriteList.begin();
		  i != SpriteList.end(); ++i )
        RenderSprite( **i, true );
}


/**
 * renders the whole map (including sprites and walls)
 */
void
CBlackFish::Render( bool SmallWalls ) const
{
	// Calculate array position at the top-left corner of the screen
	// World to array is by a simple scale by 1/32 (see the matrices in readme.txt)
	// this gives the cell at the screen center, an offset is added
	// to get the cell in the corner
	tSINT32 ArrayStartX   = static_cast<tSINT32>( WorldPos.x / 32.0 ); ArrayStartX += 3;
	tSINT32 ArrayStartY   = static_cast<tSINT32>( WorldPos.y / 32.0 ); ArrayStartY -= 13;

	// TempX is used to convert float worldcoordinates into integer coordinates,
	// so i can use the % (modulo) operator
	// TempX and TempY will become pixel-offsets later on, so I divide the world
	// coordinates by 2, loosing precision ( 2 Worldunits correspond to one pixel )
	tSINT32 TempX = static_cast<tSINT32>( WorldPos.x / 2.0 ); TempX %= 16;
	tSINT32 TempY = static_cast<tSINT32>( WorldPos.y / 2.0 ); TempY %= 16;

	// Matrix to convert from isospace to screenspace ( see readme.txt )
	tSINT32 PixelX = 2 * ( TempX + TempY );
	tSINT32 PixelY = TempY - TempX;

	// adjust pixel offsets to start in our map area and not at 0,0 screen coordinates
	// also, draw half a cell off screen (-32), so we don't see a jiggsaw pattern when
	// scrolling cells off/on screen
	tSINT32 PixelStartX      = -PixelX + m_ViewRect.left - 32;// + 12; !! 12 pixel off screen center
	tSINT32 PixelStartY		 = -PixelY + m_ViewRect.top - 32;

	RenderGround( ArrayStartX, ArrayStartY, PixelStartX, PixelStartY );

	// -1 because sprites at the left screen border will disappear too early otherwise
	RenderWallsAndSprites( ArrayStartX - 1, ArrayStartY - 1, PixelStartX - 64, PixelStartY, SmallWalls );

	DEBUGCODE
	(
	// plot a pixel at the viewport center (wrong by 12 pixels)
    CPaint Painter;
    Painter.SetDrawingTarget( RenderTarget );
    Painter.BeginDrawing();
    Painter.PutPixel((m_ViewRect.right + m_ViewRect.left) / 2 - 12,
                     (m_ViewRect.bottom + m_ViewRect.top) / 2 , 0xFFFF );
    Painter.EndDrawing();
	)
}

