#ifndef __BLACKFISH_HPP__
#define __BLACKFISH_HPP__
#include "General/Configuration.hpp"

/******************************************************************************
 * BlackFish
 * Copyright (C) 1999-2001 Sren Meyer-Eppler, Jutta Jordans
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/

#pragma warning ( push, 1 )
#ifndef __WINDOWS_H__
#define __WINDOWS_H__
  #ifndef WIN32_LEAN_AND_MEAN // !! reset this after including windows.h?
  #define WIN32_LEAN_AND_MEAN
  #endif
#include <windows.h> // !! nur fr RECT ?!
#endif
#include <list>
#include <boost/utility.hpp>
#pragma warning ( pop )

#include "BlackFish/Surface.hpp"
#include "General/Type.hpp"
#include "General/Vector.hpp"
#include "Map/Map.hpp"

class CCache;
class CTileSet;
class CSprite;


class
CBlackFish : private boost::noncopyable
{
public:
	enum tTILESETTYPE 
	{ 
		 GROUND = 0
		,OVERLAY
		,WALL
		,SPRITE
		,ALL
		,MAX 
	};
    RECT   m_ViewRect;

    CBlackFish( 
		CSurface *SystemBuffer,
		CCache *Cache,
        const char *IniFileName );
    virtual ~CBlackFish( );

    void Render( bool SmallWalls = false ) const;
	void Scroll( tDIRECTION Direction );
    void Scroll( tSINT32 PosX, tSINT32 PosY );
	void ChangeMap( const char *FileName );
    void NewMap( const CMap::tMAPHEADER & MapHeader );
    CMap *GetMap( void ) const { return Map; };

	void GetBlitRectForCell( RECT & Rect, tSINT32 ArrayX, tSINT32 ArrayY ) const;
    bool GetCellForScreen( tSINT32 & ArrayX, tSINT32 & ArrayY, tSINT32 ScreenX, tSINT32 ScreenY ) const;
    void GetScreenForWorld( tSINT32 & ScreenX, tSINT32 & ScreenY, const CVector & World ) const;
	tSINT32 GetNrOfTileSets( tTILESETTYPE Type ) const;

    inline void SetPosition( CVector *Position )
		{ WorldPos.x = Position->x; WorldPos.y = Position->y; };
	inline tSINT32 WorldX( void ) const { return static_cast<tSINT32>( WorldPos.x ); };
	inline tSINT32 WorldY( void ) const { return static_cast<tSINT32>( WorldPos.y ); };

protected:                              // protected because CEditFish inherits from this and needs access to the data
    CSurface  *RenderTarget;            // invisible videomemory surface
    CSurface   ScrapSurface;

	tUINT32    TransparentColor;

    CCache    *Cache;

    tSINT32	   NrOfTileSetsGround;
    tSINT32	   NrOfTileSetsOverlay;
    tSINT32	   NrOfTileSetsWall;
    tSINT32	   NrOfTileSetsSprite;
    tSINT32	   NrOfTileSets;
    CTileSet **Tiles;

	tSINT32    WorldWidth;
	tSINT32    WorldHeight;
    CVector    WorldPos;

    CMap      *Map;

	tSINT32	   ScrollSpeed;
	tSINT64	   ScrollTime;
    bool       Scrolling;
    CVector    ScrollDir;

    enum { TILESVISIBLEX = 11, TILESVISIBLEY = 33 };

    void RenderGround( tSINT32 ArrayStartX, tSINT32 ArrayStartY,
  					   tSINT32 PixelStartX, tSINT32 PixelStartY ) const;
	void RenderWallsAndSprites( tSINT32 ArrayStartX, tSINT32 ArrayStartY,
								tSINT32 PixelStartX, tSINT32 PixelStartY,
                                bool SmallWalls ) const;
    void RenderSprite( const std::list<CSprite *> & SpriteList, bool Dithered = false ) const;
};

#endif

