#ifndef __CACHE_HPP__
#define __CACHE_HPP__
#include "General/Configuration.hpp"

/******************************************************************************
 * BlackFish
 * Copyright (C) 1999-2001 Sren Meyer-Eppler, Jutta Jordans
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/

/******************************************************************************
    Purpose:
    A Videomemory Surface Cache using the LRU (Least Recently Used) algorithm
******************************************************************************/

#pragma warning ( disable : 4511 4512 )
#pragma warning ( disable : 4786 )
#pragma warning ( push, 1 )
#ifndef __WINDOWS_H__
#define __WINDOWS_H__
  #ifndef WIN32_LEAN_AND_MEAN // !! reset this after including windows.h?
  #define WIN32_LEAN_AND_MEAN
  #endif
#include <windows.h> // !! nur fr RECT ?!
#endif
#include <boost/utility.hpp>
#include <vector>
#pragma warning ( pop )

#include "General/Type.hpp"

class CSurface;
class CDxDraw;

class
CCache : private boost::noncopyable
{
public:
    CCache(
        tSINT32 NrOf16x16,
		tSINT32 NrOf64x32,
		tSINT32 NrOf64x128,
		tSINT32 NrOf128x128,
        tUINT32 TransparentColor );
    ~CCache();

    void Invalidate( void );
    void Invalidate( CSurface *Surface );

    void Blit( 
		const CSurface &Source, 
		const RECT *SourceRect, 
		CSurface &Dest, 
		const RECT *DestRect );

    void BlitTrans( 
		const CSurface &Source, 
		const RECT *SourceRect, 
		CSurface &Dest, 
		const RECT *DestRect );

private:
	class
	cPOOL
	{
	public:
		class
		cSLOT
		{
		public:
			RECT Rect;							// what part of the cache surface is mine?
            CSurface *CacheSurface;             // what cache surface do I belong to?
            const CSurface *SourceSurface;      // the source (systemmemory) surface    
			cPOOL *Pool;
			cSLOT *Prev;
			cSLOT *Next;

            cSLOT( const RECT &Rect, CSurface *CacheSurface, CSurface *SourceSurface, 
				cPOOL *Pool, cSLOT *Prev = NULL, cSLOT *Next = NULL );
		};
		
		enum { SurfaceWidth = 256 };   // constant, Width = Height, so only one is needed
		
		tSINT32 SlotWidth;
		tSINT32 SlotHeight;
		tSINT32 NrOfFreeSlots;
		
		cSLOT *First;
		cSLOT *Last;
        std::vector< CSurface * >   CacheSurface;
		tSINT32 OverflowCount;
		
		cPOOL( tSINT32 SlotWidth, tSINT32 SlotHeight, tSINT32 NrOfSurfaces, tUINT32 TransparentColor );
		~cPOOL( );
		
		void Invalidate( void );
		void PromoteSlot( cSLOT *Slot );
        cSLOT *CacheInSurface( const CSurface *Surface );
		
	private:
		cPOOL & operator = ( const cPOOL & rhs );
		cPOOL( const cPOOL & rhs );
	};

	const tUINT32 PoolNr16x16;
	const tUINT32 PoolNr64x32;
	const tUINT32 PoolNr64x128;
	const tUINT32 PoolNr128x128;
	std::vector<cPOOL *> Pool;	// information about the cache's videomemory surfaces

    cPOOL::cSLOT * CCache::CacheInSurface( const CSurface *Surface );
};

#endif
