#include "Dialog.hpp"

/******************************************************************************
 * BlackFish
 * Copyright (C) 1999-2001 Sren Meyer-Eppler, Jutta Jordans
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/


#pragma warning ( push, 1 )
#include <stdlib.h>
#include <fstream>
#pragma warning ( pop )

#include "general\HelpMacros.hpp"
//#include "general\Debug.hpp"
#include "BlackFish\Paint.hpp"
#include "BlackFish\Surface.hpp"
#include "BlackFish\PixelConverter.hpp"
#include "BlackFish\input.hpp"
#include "BlackFish\write.hpp"

// PCLint gives a warning (E509) here, but I do not see
// another way to do this at the moment
extern CDialog* Dialog     = NULL;

CEditBox::CEditBox(tSINT32 LeftPos,
				   tSINT32 TopPos,
				   tSINT32 MaxLength,
                   CPaint* DialogPainter,
				   tUINT16 Color)
	: Left(LeftPos)
	, Top(TopPos)
	, Length(MaxLength)
	, Position(0)
	, EditPainter(DialogPainter)
	, BackgroundColor(Color)
	, String()
{
}

CEditBox::~CEditBox()
{
	EditPainter = NULL;
}

void
CEditBox::AddChar(char ASCII)
{
	if (Position < Length)
	{
		EditPainter->Fill(Left + Position * 8,
					  Top,
					  Left + (Position + 1) * 8,
					  Top + 16,
					  BackgroundColor);
		String.replace(static_cast<tUINT32> (Position), 1, 1, ASCII);
		Writer->Out(Left + Position * 8, Top, String.substr(static_cast<tUINT32> (Position),1).c_str());
		Position++;
		SetCursor();
	}
}

void
CEditBox::DeleteLastChar()
{
	if (Position > 0)
	{
		DeleteCursor();
		Position--;
		String[static_cast<tUINT32> (Position)] = 0;
		EditPainter->Fill(Left + Position * 8,
					  Top,
					  Left + (Position + 2) * 8,
					  Top + 16,
					  BackgroundColor);
		SetCursor();
	}
}

void
CEditBox::Reset()
{
	//memset(String, 0, Length+1);
	String.erase();
	EditPainter->Fill(Left,
					  Top,
					  Left + 1 + Position * 8,
					  Top + 16,
					  BackgroundColor);
	Position = 0;
}

/*
 * assumes that String is empty and should be called directly after
 * construction
 */
void
CEditBox::Init(const char* InitString)
{
	//memcpy(String, InitString, sizeof(InitString));
	String = InitString;
	Position = 0;
	Writer->Out( Left, Top, String.c_str());
}

void
CEditBox::SetCursor()
{
	EditPainter->Fill(Left + Position * 8, Top,
					  Left + 1 + Position * 8, Top + 16,
					  0xffff);
	// hier mte dann der Timer gestartet werden etc.
}

void
CEditBox::DeleteCursor()
{
	EditPainter->Fill(Left + Position * 8, Top,
					  Left + 1 + Position * 8, Top + 16,
					  BackgroundColor);
}


// construction, position set to default if no values are passed
CDialog::CDialog( const std::string& BmpFileName, tSINT32 PosX, tSINT32 PosY )
	: FileName( BmpFileName )
	, BmpSurface( NULL )
	, PositionX( PosX )
	, PositionY( PosY )
	, Width( 0 )
	, Height( 0 )
    , DialogActive( true )
	, PixelConverter( NULL )
{
	Initialize();
}

CDialog::~CDialog()
{
	SAFEDELETE (BmpSurface);
	SAFEDELETE( PixelConverter );
    *LOG << "CDialog destroyed" << std::endl;
}

void
CDialog::Initialize()
{
    CPaint *Painter = new CPaint();

    bool Check;
	Check = Painter->GetBmpDimensions(FileName.c_str(), &Width, &Height);
	ASSERT (Check);

    BmpSurface = new CSurface();
    BmpSurface->Create(Width, Height, false);

	Painter->SetDrawingTarget(BmpSurface);
	Painter->LoadBmp(FileName.c_str());

    PixelConverter = new CPixelConverter( BmpSurface );
    ASSERT( PixelConverter != NULL );
    tUINT32 TransparentColor = PixelConverter->RGBToColor( 255, 0, 255 );
    BmpSurface->SetTransparentColor(TransparentColor);

	// bye, bye Painter
	SAFEDELETE (Painter);
}

bool
CDialog::SetPosition( tSINT32 PosX, tSINT32 PosY )
{
	// if new Position is valid for Dialog, move Dialog there
	if ((PosX < 800 - Width) &&
		(PosY < 600 - Height) &&
		(PosX >= 0) && (PosY >= 0))
	{
		PositionX = PosX;
		PositionY = PosY;
        return true;
	}
	else
	{
        return false;
	}

}


void
CDialog::MouseClick( tSINT32 MouseX, tSINT32 MouseY, tUINT32 Click)
{
	// to get rid of warning:
	MouseX = MouseX;
	MouseY = MouseY;

	// detect left mouse click
    if (Click & CMouse::Button1)
	{

	}
}

void
CDialog::CharMessage(char ASCII)
{
	*LOG << ASCII ;
}

void
CDialog::Show( CSurface* BlitSurface )
{
	tSINT32 PosX, PosY;
	GetPosition( &PosX, &PosY );
//	tUINT32 Width, Height;
//	GetSize(&Width, &Height);

	RECT Dest;
	Dest.left	= PosX;
	Dest.top	= PosY;
	Dest.right  = Dest.left + static_cast<tSINT32> (Width);
	Dest.bottom = Dest.top + static_cast<tSINT32> (Height);

	RECT Source;
	Source.left   = 0;
	Source.top    = 0;
	Source.right  = Source.left + static_cast<tSINT32> (Width);
	Source.bottom = Source.top + static_cast<tSINT32> (Height);

	BlitSurface->BlitTrans( *GetSurface(), &Source, &Dest);
}

tUINT32
CDialog::GetState()
{
	if (DialogActive)
		return ACTIVE;
	else
		return FINISH;
};

//////////////////////////////////////////////////////////////////////////////////


CNewMapDialog::CNewMapDialog()
    : CDialog("graphics\\NewMapDialog.bmp")
	, EditName( NULL )
	, EditAuthor( NULL )
	, EditWidth( NULL )
	, EditHeight( NULL )
	, Painter( NULL )
    , WidthChanged( false )
    , HeightChanged( false )
	, InputFocus( FOCUS_NONE )
	, ReturnData( NULL )
{
	Writer->SetDrawingTarget(BmpSurface);

	ReturnData = new cRETURNDATA();
	ReturnData->Width = 20;
	ReturnData->Height = 20;

    Painter = new CPaint();
	Painter->SetDrawingTarget(BmpSurface);

    EditName   = new CEditBox(32, 102, 55, Painter, 0);
    EditAuthor = new CEditBox(32, 140, 55, Painter, 0);
    EditWidth  = new CEditBox(120, 180, 3, Painter, 0);
	EditWidth->Init("20");
    EditHeight = new CEditBox(272, 180, 3, Painter, 0);
	EditHeight->Init("20");

	ChangeInputFocus(FOCUS_NAME);
}

CNewMapDialog::~CNewMapDialog()
{
	SAFEDELETE( EditName );
	SAFEDELETE( EditAuthor );
	SAFEDELETE( EditWidth );
	SAFEDELETE( EditHeight );
	SAFEDELETE( Painter );
	SAFEDELETE( ReturnData );
}

void
CNewMapDialog::CharMessage(char ASCII)
{
	Writer->SetDrawingTarget( BmpSurface );
	if (ASCII == '\t')
	{
		ChangeInputFocus();
	}
	// shift tab
	else if (ASCII == '\v')
	{
		ChangeInputFocus(FOCUS_BACK);
	}
	else
	{
		switch (InputFocus)
		{
			case FOCUS_NAME :
				if (ASCII == 13)
				{
					EditName->DeleteLastChar();
				}
				else
				{
					EditName->AddChar(ASCII);
				}
			break;
			case FOCUS_AUTHOR :
				if (ASCII == 13)
				{
					EditAuthor->DeleteLastChar();
				}
				else
				{
					EditAuthor->AddChar(ASCII);
				}
			break;
			case FOCUS_WIDTH :
				if (ASCII == 13)
				{
					EditWidth->DeleteLastChar();
				}
				else if ((ASCII >= 48) && (ASCII <= 57))
				{
					EditWidth->AddChar(ASCII);
				}
			break;
			case FOCUS_HEIGHT :
				if (ASCII == 13)
				{
					EditHeight->DeleteLastChar();
				}
				else if ((ASCII >= 48) && (ASCII <= 57))
				{
					EditHeight->AddChar(ASCII);
				}
			break;
			case FOCUS_OK :
				if (ASCII == '\n')
				{
					OnOk();
				}
			break;
			case FOCUS_CANCEL :
				if (ASCII == '\n')
				{
					OnCancel();
				}
			break;
			default :
				// should not happen
				ASSERT(0);
		}
	}
}

void
CNewMapDialog::MouseClick(tSINT32 MouseX, tSINT32 MouseY, tUINT32 Click)
{
	// all left clicks handled here
    if (Click & CMouse::Button1)
	{
		// check if MousePointer is over OK Button
		if ((MouseX >= 400 + PositionX) && (MouseX <= 456 + PositionX) &&
			(MouseY >= 173 + PositionY) && (MouseY <= 198 + PositionY))
		{
			OnOk();
		}

		//check if Mouse is over Cancel button
		if ((MouseX >= 460 + PositionX) && (MouseX <= 516 + PositionX) &&
			(MouseY >= 174 + PositionY) && (MouseY <= 196 + PositionY))
		{
			OnCancel();
		}

		// check if Mouse is over Name Field
		if ((MouseX >= 28 + PositionX) && (MouseX <= 516 + PositionX) &&
			(MouseY >= 100 + PositionY) && (MouseY <= 120 + PositionY))
		{
			ChangeInputFocus(FOCUS_NAME);
		}
		// check if Mouse is over Author Field
		else if ((MouseX >= 20 + PositionX) && (MouseX <= 512 + PositionX) &&
				 (MouseY >= 138 + PositionY) && (MouseY <= 160 + PositionY))
		{
			ChangeInputFocus(FOCUS_AUTHOR);
		}
		// check if Mouse is over Width Field
		else if ((MouseX >= 110 + PositionX) && (MouseX <= 174 + PositionX) &&
				 (MouseY >= 178 + PositionY) && (MouseY <= 196 + PositionY))
		{

			ChangeInputFocus(FOCUS_WIDTH);
		}
		// check if Mouse is over Height Field
		else if ((MouseX >= 262 + PositionX) && (MouseX <= 320 + PositionX) &&
				 (MouseY >= 174 + PositionY) && (MouseY <= 194 + PositionY))
		{

			ChangeInputFocus(FOCUS_HEIGHT);
		}
	}
}

void
CNewMapDialog::ChangeInputFocus(tFOCUS NewFocus)
{
	if (NewFocus == InputFocus)
		return;

	tUINT16 Green = 0;
	if ( PixelConverter )
	{
		Green = PixelConverter->RGBToColor(0,69,0);
	}
	else
	{
		ASSERT( 0 );
	}

	switch (InputFocus)
	{
		case FOCUS_NAME :
			EditName->DeleteCursor();
			if (NewFocus == FOCUS_NONE)
				NewFocus = FOCUS_AUTHOR;
			else if (NewFocus == FOCUS_BACK)
				NewFocus = FOCUS_CANCEL;
		break;
		case FOCUS_AUTHOR :
			EditAuthor->DeleteCursor();
			if (NewFocus == FOCUS_NONE)
				NewFocus = FOCUS_WIDTH;
			else if (NewFocus == FOCUS_BACK)
				NewFocus = FOCUS_NAME;
		break;
		case FOCUS_WIDTH :
			EditWidth->DeleteCursor();
			if (NewFocus == FOCUS_NONE)
				NewFocus = FOCUS_HEIGHT;
			else if (NewFocus == FOCUS_BACK)
				NewFocus = FOCUS_AUTHOR;
		break;
		case FOCUS_HEIGHT :
			EditHeight->DeleteCursor();
			if (NewFocus == FOCUS_NONE)
				NewFocus = FOCUS_OK;
			else if (NewFocus == FOCUS_BACK)
				NewFocus = FOCUS_WIDTH;
		break;
		case FOCUS_OK :
			Painter->SetDrawingTarget(BmpSurface);
			Painter->BeginDrawing();
			Painter->DrawRect(401, 173, 457, 198, Green);
			Painter->EndDrawing();
			if (NewFocus == FOCUS_NONE)
				NewFocus = FOCUS_CANCEL;
			else if (NewFocus == FOCUS_BACK)
				NewFocus = FOCUS_HEIGHT;
		break;
		case FOCUS_CANCEL :
			Painter->SetDrawingTarget(BmpSurface);
			Painter->BeginDrawing();
			Painter->DrawRect(461, 173, 516, 198, Green);
			Painter->EndDrawing();
			if (NewFocus == FOCUS_NONE)
				NewFocus = FOCUS_NAME;
			else if (NewFocus == FOCUS_BACK)
				NewFocus = FOCUS_OK;
		break;
		default :
			// no need to do anything, can happen only on initialisation
		break;
	}

	switch (NewFocus)
	{
		case FOCUS_NAME :
			InputFocus = FOCUS_NAME;
			EditName->SetCursor();
		break;
		case FOCUS_AUTHOR :
			InputFocus = FOCUS_AUTHOR;
			EditAuthor->SetCursor();
		break;
		case FOCUS_HEIGHT :
			if(!HeightChanged)
			{
				//EditHeight->Reset();
                HeightChanged = true;
			}
			InputFocus = FOCUS_HEIGHT;
			EditHeight->SetCursor();
		break;
		case FOCUS_WIDTH :
			if(!WidthChanged)
			{
				//EditWidth->Reset();
                WidthChanged = true;
			}
			InputFocus = FOCUS_WIDTH;
			EditWidth->SetCursor();
		break;
		case FOCUS_OK :
			InputFocus = FOCUS_OK;
			Painter->SetDrawingTarget(BmpSurface);
			Painter->BeginDrawing();
			Painter->DrawRect( 401, 173, 457, 198, 0 );
			Painter->EndDrawing();
		break;
		case FOCUS_CANCEL :
			InputFocus = FOCUS_CANCEL;
			Painter->SetDrawingTarget(BmpSurface);
			Painter->BeginDrawing();
			Painter->DrawRect( 461, 173, 516, 198, 0 );
			Painter->EndDrawing();
		break;
		default :
			//should not happen
			ASSERT( 0 );
	}
}

void
CNewMapDialog::OnOk()
{
	tUINT32 MapWidth = static_cast<tUINT32> (atol(EditWidth->GetString().c_str()));
	tUINT32 MapHeight = static_cast<tUINT32> (atol(EditHeight->GetString().c_str()));
	if((MapWidth > 0) && (MapWidth <= 512))
	{
		ReturnData->Width = MapWidth;
	}
	if((MapHeight > 0) && (MapHeight <= 512))
	{
		ReturnData->Height = MapHeight;
	}
	ReturnData->Name = EditName->GetString().c_str();
	ReturnData->Author = EditAuthor->GetString().c_str();
    DialogActive = false;
}

void
CNewMapDialog::OnCancel()
{
	SAFEDELETE(ReturnData);
    DialogActive = false;
}

CDialog::cRETURNDATABASE*
CNewMapDialog::GetDataPackage()
{
	return ReturnData;
}

CLoadMapDialog::CLoadMapDialog()
    : CDialog("graphics\\LoadMapDialog.bmp")
	, EditFileName( NULL )
	, Painter( NULL )
	, InputFocus( FOCUS_NONE )
	, ReturnData( NULL )
{
    Painter = new CPaint();
	Painter->SetDrawingTarget(BmpSurface);
    EditFileName = new CEditBox( 32, 63, 50, Painter, 0);
	ReturnData = new cRETURNDATA();
	ChangeInputFocus(FOCUS_FILE);
}

CLoadMapDialog::~CLoadMapDialog()
{
	SAFEDELETE(EditFileName);
	SAFEDELETE(ReturnData);
	SAFEDELETE(Painter);
}

CDialog::cRETURNDATABASE*
CLoadMapDialog::GetDataPackage()
{
	return ReturnData;
}

void
CLoadMapDialog::CharMessage(char ASCII)
{
	Writer->SetDrawingTarget(BmpSurface);
	if (ASCII == '\t')
	{
		ChangeInputFocus();
	}
	//shift tab
	else if (ASCII == '\v')
	{
		ChangeInputFocus(FOCUS_BACK);
	}
	else
	{
		switch (InputFocus)
		{
			case FOCUS_FILE :
				if (ASCII == 13)
				{
					EditFileName->DeleteLastChar();
				}
				else
				{
					EditFileName->AddChar(ASCII);
				}
			break;
			case FOCUS_OK :
				if (ASCII == '\n')
				{
					OnOk();
				}
			break;
			case FOCUS_CANCEL :
				if (ASCII == '\n')
				{
					OnCancel();
				}
			break;
			default :
				// should not happen
				ASSERT( 0 );
		}
	}
}

void
CLoadMapDialog::MouseClick(tSINT32 MouseX, tSINT32 MouseY, tUINT32 Click)
{
	// detect left click on ok button
	if ((MouseX >= 360 + PositionX) && (MouseX < 416 + PositionY) &&
		(MouseY >= 84  + PositionX) && (MouseY < 108 + PositionY) &&
        (Click & CMouse::Button1))
	{
		OnOk();
	}
	//detect left click on cancel button
	if ((MouseX >= 420 + PositionX) && (MouseX <= 476 + PositionX) &&
		(MouseY >= 84 + PositionY)  && (MouseY <= 108 + PositionY) &&
        (Click & CMouse::Button1))
	{
		OnCancel();
	}
}

void
CLoadMapDialog::OnOk()
{
	std::string TestFileName = "maps\\";
	TestFileName.append(EditFileName->GetString());
	if (TestFileName.find(".map") == std::string::npos)
	{
		TestFileName.append(".map");
	}

*LOG << FileName.c_str() << std::endl;

	std::fstream TestStream;
	TestStream.open(TestFileName.c_str(), std::ios::in|std::ios::binary);
	if(TestStream.is_open())
	{
		ReturnData->FileName = TestFileName;
        DialogActive = false;
		TestStream.close();
	}
	else
	{
		EditFileName->Reset();
		ChangeInputFocus(FOCUS_FILE);
		EditFileName->SetCursor();
	}
}

void
CLoadMapDialog::OnCancel()
{
	SAFEDELETE(ReturnData);
    DialogActive = false;
}

void
CLoadMapDialog::ChangeInputFocus(tFOCUS NewFocus)
{
	if (InputFocus == NewFocus)
		return;

	tUINT16 Green = 0;
	if ( PixelConverter )
	{
		Green = PixelConverter->RGBToColor(0,69,0);
	}
	else
	{
		ASSERT( 0 );
	}

	switch (InputFocus)
	{
		case FOCUS_FILE :
			if (NewFocus == FOCUS_NONE)
				NewFocus = FOCUS_OK;
			else if (NewFocus == FOCUS_BACK)
				NewFocus = FOCUS_CANCEL;
			EditFileName->DeleteCursor();
		break;
		case FOCUS_OK :
			if (NewFocus == FOCUS_NONE)
				NewFocus = FOCUS_CANCEL;
			else if (NewFocus == FOCUS_BACK)
				NewFocus = FOCUS_FILE;
			Painter->SetDrawingTarget(BmpSurface);
			Painter->BeginDrawing();
			Painter->DrawRect(360, 84, 416, 109, Green);
			Painter->EndDrawing();
		break;
		case FOCUS_CANCEL :
			if (NewFocus == FOCUS_NONE)
				NewFocus = FOCUS_FILE;
			else if (NewFocus == FOCUS_BACK)
				NewFocus = FOCUS_OK;
			Painter->SetDrawingTarget(BmpSurface);
			Painter->BeginDrawing();
			Painter->DrawRect(420, 84, 476, 109, Green);
			Painter->EndDrawing();
		break;
		default :
			// no need to do anything, can happen only on initialisation
		break;
	}

	switch (NewFocus)
	{
		case FOCUS_FILE :
			InputFocus = FOCUS_FILE;
			EditFileName->SetCursor();
		break;
		case FOCUS_OK :
			InputFocus = FOCUS_OK;
			Painter->SetDrawingTarget(BmpSurface);
			Painter->BeginDrawing();
			Painter->DrawRect(360, 84, 416, 109, 0);
			Painter->EndDrawing();
		break;
		case FOCUS_CANCEL :
			InputFocus = FOCUS_CANCEL;
			Painter->SetDrawingTarget(BmpSurface);
			Painter->BeginDrawing();
			Painter->DrawRect(420, 84, 476, 109, 0);
			Painter->EndDrawing();
		break;
		default :
			//should not happen
			ASSERT( 0 );
	}
}

CSaveMapDialog::CSaveMapDialog(const char* MapName, const char* Author, std::string MapFileName)
    : CDialog("graphics\\SaveMapDialog.bmp")
	, EditName( NULL )
	, EditAuthor( NULL )
	, EditFile( NULL )
	, Painter( NULL )
    , NameChanged( false )
    , AuthorChanged( false )
    , FileChanged( false )
	, InputFocus( FOCUS_NONE )
	, ReturnData( NULL )
{
	Writer->SetDrawingTarget(BmpSurface);
    Painter = new CPaint();
	Painter->SetDrawingTarget(BmpSurface);

    EditName = new CEditBox(32, 80, 55, Painter, 0);
	EditName->Init(MapName);
    EditAuthor = new CEditBox(32, 126, 55, Painter, 0);
	EditAuthor->Init(Author);
    EditFile = new CEditBox(32, 170, 55, Painter, 0);
	if(MapFileName != "")
	{
		//desect file name string so that only file title remains
		tUINT32 StartPos = MapFileName.find_last_of("\\") + 1;
		tUINT32 EndPos	 = MapFileName.find_last_of(".");
		MapFileName = MapFileName.substr(StartPos, EndPos - StartPos);
	}
	EditFile->Init(MapFileName.c_str());

	ChangeInputFocus(FOCUS_OK);

	ReturnData = new cRETURNDATA();
	ReturnData->MapName = MapName;
	ReturnData->Author  = Author;
	ReturnData->FileName = "maps\\";
}

CSaveMapDialog::~CSaveMapDialog()
{
	SAFEDELETE(Painter);
	SAFEDELETE(EditName);
	SAFEDELETE(EditAuthor);
	SAFEDELETE(EditFile);
	SAFEDELETE(ReturnData);
}

void
CSaveMapDialog::CharMessage(char ASCII)
{
	Writer->SetDrawingTarget(BmpSurface);

	if (ASCII == '\t')
	{
		ChangeInputFocus();
	}
	//shift tab
	else if (ASCII == '\v')
	{
		ChangeInputFocus(FOCUS_BACK);
	}
	else
	{
		switch(InputFocus)
		{
			case FOCUS_NAME :
				if (ASCII == 13)
				{
					EditName->DeleteLastChar();
				}
				else
				{
					EditName->AddChar(ASCII);
				}
			break;
			case FOCUS_AUTHOR :
				if (ASCII == 13)
				{
					EditAuthor->DeleteLastChar();
				}
				else
				{
					EditAuthor->AddChar(ASCII);
				}
			break;
			case FOCUS_FILE :
				if (ASCII == 13)
				{
					EditFile->DeleteLastChar();
				}
				else
				{
					EditFile->AddChar(ASCII);
				}
			break;
			case FOCUS_OK :
				if (ASCII == '\n')
				{
					OnOk();
				}
			break;
			case FOCUS_CANCEL :
				if (ASCII == '\n')
				{
					OnCancel();
				}
			break;
			default :
				//should not happen
				ASSERT( 0 );
		}
	}
}

void
CSaveMapDialog::MouseClick(tSINT32 MouseX, tSINT32 MouseY, tUINT32 Click)
{
	// all left clicks
    if (Click & CMouse::Button1)
	{
		// Map Name field
		if ((MouseX >= 22 + PositionX) && (MouseX <= 484 + PositionX) &&
			(MouseY >= 76 + PositionY) && (MouseY <= 96  + PositionY))
		{
			ChangeInputFocus(FOCUS_NAME);
		}
		// Author field
		else if ((MouseX >= 22  + PositionX) && (MouseX <= 484 + PositionX) &&
			     (MouseY >= 122 + PositionY) && (MouseY <= 142 + PositionY))
		{
			ChangeInputFocus(FOCUS_AUTHOR);
		}
		// FileName field
		else if ((MouseX >= 22  + PositionX) && (MouseX <= 484 + PositionX) &&
			     (MouseY >= 166 + PositionY) && (MouseY <= 186 + PositionY))
		{
			ChangeInputFocus(FOCUS_FILE);
		}
		// OK Button
		else if ((MouseX >= 352 + PositionX) && (MouseX <= 408 + PositionX) &&
			     (MouseY >= 196 + PositionY) && (MouseY <= 220 + PositionY))
		{
			OnOk();
		}
		// Cancel Button
		else if ((MouseX >= 412 + PositionX) && (MouseX <= 468 + PositionX) &&
			     (MouseY >= 196 + PositionY) && (MouseY <= 220 + PositionY))
		{
			OnCancel();
		}
	}
}

void
CSaveMapDialog::OnOk()
{
	if(NameChanged)
	{
		ReturnData->MapName = EditName->GetString();
	}
	if(AuthorChanged)
	{
		ReturnData->Author = EditAuthor->GetString();
	}
	ReturnData->FileName.append(EditFile->GetString());
	if (ReturnData->FileName.find(".map") == std::string::npos)
	{
		ReturnData->FileName.append(".map");
	}

	std::ofstream TestStream;
	TestStream.open(ReturnData->FileName.c_str(), std::ios_base::binary);
	if (TestStream.is_open())
	{
        DialogActive = false;
		TestStream.close();
	}
	else // invalid outputfile
	{
		*LOG << "Opening of TestStream failed" << std::endl;
		ChangeInputFocus(FOCUS_FILE);
		EditFile->SetCursor();
	}
}

void
CSaveMapDialog::OnCancel()
{
	SAFEDELETE(ReturnData);
    DialogActive = false;
}

void
CSaveMapDialog::ChangeInputFocus(tFOCUS NewFocus)
{
	if (NewFocus == InputFocus)
		return;

	tUINT16 Green = 0;
	if ( PixelConverter )
	{
		Green = PixelConverter->RGBToColor(0,69,0);
	}
	else
	{
		ASSERT( 0 );
	}

	switch (InputFocus)
	{
		case FOCUS_NAME :
			EditName->DeleteCursor();
			if(NewFocus == FOCUS_NONE)
				NewFocus = FOCUS_AUTHOR;
			else if (NewFocus == FOCUS_BACK)
				NewFocus = FOCUS_CANCEL;
		break;
		case FOCUS_AUTHOR :
			EditAuthor->DeleteCursor();
			if(NewFocus == FOCUS_NONE)
				NewFocus = FOCUS_FILE;
			else if (NewFocus == FOCUS_BACK)
				NewFocus = FOCUS_NAME;
		break;
		case FOCUS_FILE :
			EditFile->DeleteCursor();
			if (NewFocus == FOCUS_NONE)
				NewFocus = FOCUS_OK;
			else if (NewFocus == FOCUS_BACK)
				NewFocus = FOCUS_AUTHOR;
		break;
		case FOCUS_OK :
			Painter->SetDrawingTarget(BmpSurface);
			Painter->BeginDrawing();
			Painter->DrawRect(352, 197, 409, 222, Green);
			Painter->EndDrawing();
			if (NewFocus == FOCUS_NONE)
				NewFocus = FOCUS_CANCEL;
			else if (NewFocus == FOCUS_BACK)
				NewFocus = FOCUS_FILE;
		break;
		case FOCUS_CANCEL :
			Painter->SetDrawingTarget(BmpSurface);
			Painter->BeginDrawing();
			Painter->DrawRect(413, 197, 468, 222, Green);
			Painter->EndDrawing();
			if (NewFocus == FOCUS_NONE)
				NewFocus = FOCUS_NAME;
			else if (NewFocus == FOCUS_BACK)
				NewFocus = FOCUS_OK;
		break;
		default :
			// no need to do anything, can happen only on initialisation
		break;
	}

	switch (NewFocus)
	{
		case FOCUS_NAME :
			InputFocus = FOCUS_NAME;
			if(!NameChanged)
			{
				//EditName->Reset();
                NameChanged = true;
			}
			EditName->SetCursor();
		break;
		case FOCUS_AUTHOR :
			InputFocus = FOCUS_AUTHOR;
			if(!AuthorChanged)
			{
				//EditAuthor->Reset();
                AuthorChanged = true;
			}
			EditAuthor->SetCursor();
		break;
		case FOCUS_FILE :
			InputFocus = FOCUS_FILE;
			if(!FileChanged)
			{
				//EditFile->Reset();
                FileChanged = true;
			}
			EditFile->SetCursor();
		break;
		case FOCUS_OK :
			InputFocus = FOCUS_OK;
			Painter->SetDrawingTarget(BmpSurface);
			Painter->BeginDrawing();
			Painter->DrawRect(352, 197, 409, 222, 0);
			Painter->EndDrawing();
		break;
		case FOCUS_CANCEL :
			InputFocus = FOCUS_CANCEL;
			Painter->SetDrawingTarget(BmpSurface);
			Painter->BeginDrawing();
			Painter->DrawRect(413, 197, 468, 222, 0);
			Painter->EndDrawing();
		break;
		default :
			// should not happen
			ASSERT( 0 );
	}
}

CDialog::cRETURNDATABASE*
CSaveMapDialog::GetDataPackage()
{
	return ReturnData;
}

