#ifndef __DIALOG_HPP__
#define __DIALOG_HPP__

/******************************************************************************
 * BlackFish
 * Copyright (C) 1999-2001 Sren Meyer-Eppler, Jutta Jordans
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/

/*********************************************************************
		Header for all Dialog classes

		History of changes:
		- File created by Capella 03/30/2000
***********************************************************************/
#ifndef __TYPE_HPP__
#include "general\type.hpp"
#endif

#pragma warning ( push, 1 )
#include <string>
#pragma warning ( pop )

class CSurface;
class CWrite;
class CPaint;
class CPixelConverter;

class CEditBox
{
private:
	//members
	tSINT32		Left;
	tSINT32		Top;
	tSINT32		Length;
	tSINT32		Position;
    CPaint*     EditPainter;
	tUINT16		BackgroundColor;
	std::string	String;

public:
    CEditBox(tSINT32 LeftPos,
			 tSINT32 TopPos,
			 tSINT32 MaxLength,
             CPaint* DialogPainter,
			 tUINT16 Color);
    ~CEditBox();
	void AddChar(char ASCII);
	void DeleteLastChar();
	std::string GetString() {return String;};
	void Reset();
	void Init(const char* InitString);
	void SetCursor();
	void DeleteCursor();
};


class CDialog
{
protected:
	//members
	std::string FileName;
    CSurface* BmpSurface;
	tSINT32 PositionX, PositionY;
	tSINT32 Width, Height;
    bool DialogActive;

    CPixelConverter* PixelConverter;

	//methods
	void Initialize();

public:
	class cRETURNDATABASE
	{
	public :
		virtual ~cRETURNDATABASE(){};
	};
    //CDialog();    default construction not allowed
    explicit CDialog( const std::string& BmpFileName,
					  tSINT32 PosX = 100,
					  tSINT32 PosY = 100);
    virtual ~CDialog();

	enum {FINISH = 0, ACTIVE = 1};

	//methods
    CSurface* GetSurface() {return BmpSurface;};
	void GetPosition(tSINT32* PosX, tSINT32* PosY) {*PosX = PositionX; *PosY = PositionY;};
	void GetSize(tSINT32* SizeX, tSINT32* SizeY) {*SizeX = Width; *SizeY = Height;};

    void Show(CSurface* BlitSurface);
    bool SetPosition( tSINT32 PosX, tSINT32 PosY);
	tUINT32 GetState();
	virtual void MouseClick ( tSINT32 MouseX, tSINT32 MouseY, tUINT32 Click);
	virtual void CharMessage ( char ASCII );
	virtual cRETURNDATABASE* GetDataPackage() = 0;
};



/*
 *	input dialog for the creation of new map, shall get name, author and
 *  dimensions from the User
 */
class CNewMapDialog : public CDialog
{
private:
	//members
    CEditBox* EditName;
    CEditBox* EditAuthor;
    CEditBox* EditWidth;
    CEditBox* EditHeight;
    CPaint*   Painter;
    bool      WidthChanged;
    bool      HeightChanged;

	enum tFOCUS
	{
		FOCUS_BACK,
		FOCUS_NONE,
		FOCUS_NAME,
		FOCUS_AUTHOR,
		FOCUS_WIDTH,
		FOCUS_HEIGHT,
		FOCUS_OK,
		FOCUS_CANCEL
	} InputFocus;

	void ChangeInputFocus( tFOCUS NewFocus = FOCUS_NONE );
	void OnOk();
	void OnCancel();

	// forbid assignement + copy construction
    CNewMapDialog( const CNewMapDialog& ):CDialog(""){};
    CNewMapDialog &operator = ( const CNewMapDialog& ) {return *this;};

public:
    CNewMapDialog();
    ~CNewMapDialog();

    class cRETURNDATA : public CDialog::cRETURNDATABASE
	{
	public :
		std::string Name;
		std::string Author;
		tSINT32 Width;
		tSINT32 Height;
	};

	//methods
    void CharMessage(char ASCII);   //overwrites CDialog method (I hope!)
    CDialog::cRETURNDATABASE* GetDataPackage();
	void MouseClick(tSINT32 MouseX, tSINT32 MouseY, tUINT32 Click);

private:
    CNewMapDialog::cRETURNDATA* ReturnData;
};

class CLoadMapDialog : public CDialog
{
private:
    CEditBox* EditFileName;
    CPaint* Painter;

	enum tFOCUS
	{
		FOCUS_BACK,
		FOCUS_NONE,
		FOCUS_FILE,
		FOCUS_OK,
		FOCUS_CANCEL
	} InputFocus;

	void ChangeInputFocus (tFOCUS NewFocus = FOCUS_NONE);
	void OnOk();
	void OnCancel();

	//forbid assignement + copy construction
    CLoadMapDialog( const CLoadMapDialog& ):CDialog(""){};
    CLoadMapDialog &operator = (const CLoadMapDialog&) {return *this;};

public:
    CLoadMapDialog();
    ~CLoadMapDialog();

    class cRETURNDATA : public CDialog::cRETURNDATABASE
	{
	public :
		std::string FileName;
	};

	//methods
	void CharMessage(char ASCII);
    CDialog::cRETURNDATABASE* GetDataPackage();
	void MouseClick(tSINT32 MouseX, tSINT32 MouseY, tUINT32 Click);

private :
	cRETURNDATA* ReturnData;

};


class CSaveMapDialog : public CDialog
{
private :
    CEditBox* EditName;
    CEditBox* EditAuthor;
    CEditBox* EditFile;
    CPaint*   Painter;
    bool      NameChanged;
    bool      AuthorChanged;
    bool      FileChanged;
	enum tFOCUS
	{
		FOCUS_BACK,
		FOCUS_NONE,
		FOCUS_NAME,
		FOCUS_AUTHOR,
		FOCUS_FILE,
		FOCUS_OK,
		FOCUS_CANCEL
	} InputFocus;

	void ChangeInputFocus(tFOCUS NewFocus = FOCUS_NONE);
	void OnOk();
	void OnCancel();

	//forbid assignement + copy construction
    CSaveMapDialog( const CSaveMapDialog& ):CDialog(""){};
    CSaveMapDialog &operator = ( const CSaveMapDialog& ) {return *this;};

public :
    CSaveMapDialog(const char* MapName, const char* Author, std::string FileName);
    ~CSaveMapDialog();

    class cRETURNDATA : public CDialog::cRETURNDATABASE
	{
	public :
		std::string MapName;
		std::string Author;
		std::string FileName;
	};

	//methods
	void CharMessage(char ASCII);
    CDialog::cRETURNDATABASE* GetDataPackage();
	void MouseClick(tSINT32 MouseX, tSINT32 MouseY, tUINT32 Click);

private :
	cRETURNDATA* ReturnData;
};

extern CDialog* Dialog;

#endif
