#include "dxdraw.hpp"

/******************************************************************************
 * BlackFish
 * Copyright (C) 1999-2001 Sren Meyer-Eppler, Jutta Jordans
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/


#pragma warning ( push, 1 )
#define DIRECTINPUT_VERSION 0x0700
#include <ddraw.h>
#pragma warning ( pop )

CDxDraw *DirectDraw = NULL;

/*
 * initializes the default 800*600*16bit Resolution, either windowed or fullscreen mode
 */
CDxDraw::CDxDraw( const HWND &GameWindow, bool FullScreen )
	: DirectDraw( NULL )
	 ,Window( 0 )
     ,FullScreen( true )
{
    Initialize( GameWindow, 800, 600, 16, FullScreen );
}

/*
 * either initializes to the default resolution and bitdepth in fullscreen mode or
 * to the mode described by the supplied parameters
 */
CDxDraw::CDxDraw(
	const HWND &GameWindow,
	tSINT32 XResolution = 800,
	tSINT32 YResolution = 600,
	tSINT32 BitDepth	= 16,
    bool FullScreen = true )
	: DirectDraw( NULL )
	 ,Window( 0 )
     ,FullScreen( true )
{
    Initialize( GameWindow, XResolution, YResolution, BitDepth, FullScreen );
}

/*
 *
 */
CDxDraw::~CDxDraw()
{
    RELEASE( DirectDraw );
}

/*
 * provate function, only called from the constructors
 */
void
CDxDraw::Initialize(
	const HWND &GameWindow,
	tSINT32 XResolution,
	tSINT32 YResolution,
	tSINT32 BitDepth,
    bool FullScreen )
{
    HRESULT ReturnValue = DD_OK;

	this->FullScreen	= FullScreen;
	Window				= GameWindow;

	ReturnValue = DirectDrawCreateEx( NULL, reinterpret_cast<void **>( &DirectDraw ), IID_IDirectDraw7, NULL);
	ASSERT( ReturnValue == DD_OK );

	if ( FullScreen )
	{
	    ReturnValue = DirectDraw->SetCooperativeLevel( GameWindow, DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN );
		ASSERT( ReturnValue == DD_OK );

		ReturnValue = DirectDraw->SetDisplayMode(
			static_cast<DWORD>( XResolution ), static_cast<DWORD>( YResolution ), static_cast<DWORD>( BitDepth), 0, 0 );
		ASSERT( ReturnValue == DD_OK );
	}
	else
	{
        ReturnValue = DirectDraw->SetCooperativeLevel( GameWindow, DDSCL_NORMAL );
        ASSERT( ReturnValue == DD_OK );
	}
}

/*
 * returns the total videomemory on the graphics card
 */
tSINT32
CDxDraw::GetTotalVideoMemory( void )
{
    ASSERT( DirectDraw != NULL );

    DDCAPS DeviceCaps;

    memset( &DeviceCaps, 0, sizeof( DeviceCaps ));
    DeviceCaps.dwSize = sizeof( DeviceCaps );

    HRESULT ReturnValue = DD_OK;
    ReturnValue = DirectDraw->GetCaps( &DeviceCaps, NULL );
    ASSERT( ReturnValue == DD_OK );

    return static_cast<tSINT32>( DeviceCaps.dwVidMemTotal );
}

/*
 * returns the total available videomemory on the graphics card
 */
tSINT32
CDxDraw::GetFreeVideoMemory( void )
{
    ASSERT( DirectDraw != NULL );

    DDCAPS DeviceCaps;

    memset( &DeviceCaps, 0, sizeof( DeviceCaps ));
    DeviceCaps.dwSize = sizeof( DeviceCaps );

    HRESULT ReturnValue = DD_OK;
    ReturnValue = DirectDraw->GetCaps( &DeviceCaps, NULL );
    ASSERT( ReturnValue == DD_OK );

    return static_cast<tSINT32>( DeviceCaps.dwVidMemFree );
}
