#ifndef __DXDRAW_HPP__
#define __DXDRAW_HPP__
#include "General/Configuration.hpp"

/******************************************************************************
 * BlackFish
 * Copyright (C) 1999-2001 Sren Meyer-Eppler, Jutta Jordans
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/

/******************************************************************************
    Purpose:
    DirectDraw Interface, responsible for setting the videomode
    only one instance per program is allowed
    needed to create surfaces later on
******************************************************************************/

#pragma warning ( push, 1 )
#ifndef __WINDOWS_H__
#define __WINDOWS_H__
  #ifndef WIN32_LEAN_AND_MEAN
  #define WIN32_LEAN_AND_MEAN
  #endif
#include <windows.h>
#endif
#pragma warning ( pop )

#ifndef __TYPE_HPP__
#include "general\type.hpp"
#endif

#ifndef __HELPMACROS_HPP__
#include "general\HelpMacros.hpp"
#endif

struct IDirectDraw7;

class
CDxDraw
{
public:

    CDxDraw( const HWND &GameWindow, bool FullScreen );
    CDxDraw( const HWND &GameWindow, tSINT32 XResolution, tSINT32 YResolution, tSINT32 BitDepth, bool FullScreen );

    ~CDxDraw();

    IDirectDraw7 *GetDirectDrawInterface( void ) { return DirectDraw; };
    bool IsFullScreen( void ) { return FullScreen; };
	HWND GetWindow( void ) { return Window; };
    tSINT32 GetTotalVideoMemory( void );
    tSINT32 GetFreeVideoMemory( void );

private:

    IDirectDraw7 *DirectDraw; // the main DirectDraw Object (the graphics hardware)
	HWND Window;
    bool FullScreen;

    void Initialize( const HWND &GameWindow, tSINT32 XResolution,
        tSINT32 YResolution, tSINT32 BitDepth, bool FullScreen );

	// copy and assignment is not allowed
    CDxDraw( const CDxDraw &DxDraw );
    CDxDraw & operator = ( const CDxDraw &DxDraw );
};

extern CDxDraw *DirectDraw;

#endif
