#ifndef __GAME_HPP__
#define __GAME_HPP__
#include "General/Configuration.hpp"

/******************************************************************************
 * BlackFish
 * Copyright (C) 1999-2001 Sren Meyer-Eppler, Jutta Jordans
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/

/******************************************************************************
    Purpose:

    !!! the ratio of message system maintenance code to real code is too high
    can I have a function in CEvent that returns the correctly cast type?
    i.e. CEvent->GetEvent( CMouseMoveEvent ) returns the correct child class or
	NULL if it can't be cast? what would the return value have to be without
	requiring another cast? are templates usefull here?
******************************************************************************/

#pragma warning ( push, 1 )
#include <vector>
#include <list>

#ifndef __WINDOWS_H__
#define __WINDOWS_H__
  #ifndef WIN32_LEAN_AND_MEAN // !! reset this after including windows.h?
  #define WIN32_LEAN_AND_MEAN
  #endif
#include <windows.h> // !! nur fr RECT ?!
#endif
#pragma warning ( pop )

#ifndef __TYPE_HPP__
#include "general\type.hpp"
#endif

#ifndef __VECTOR_HPP__
#include "general\vector.hpp"
#endif

class CSurface;
class CTileSet;
class CDxDraw;
class CSprite;
class CCache;
class CMap;
class CPaint;
class CInput;
class CKeyboard;
class CMouse;
class CBlackFish;
class CMarineSprite;
class CBobbingSprite;

class
CEvent
{
public:
    explicit CEvent( bool IsPersistant = false ) : IsPersistant( IsPersistant ) { };
    virtual ~CEvent( ) = 0 { };

    bool Persistant( void ) { return IsPersistant; };
private:
    bool IsPersistant;
};

class
CMouseClickEvent : public CEvent
{
public:
	tSINT32 PositionX;
	tSINT32 PositionY;
	tSINT32 MouseWheel;
	tUINT32 Buttons;
	tUINT32 LastClick;

    CMouseClickEvent(
		tSINT32 PositionX,
		tSINT32 PositionY,
		tSINT32 MouseWheel,
		tUINT32 Buttons,
		tUINT32 LastClick )
        : CEvent( false ),
		  PositionX( PositionX ),
		  PositionY( PositionY ),
		  MouseWheel( MouseWheel ),
		  Buttons( Buttons ),
		  LastClick( LastClick )
	{ };
};

class
CMouseMoveEvent : public CEvent
{
public:
	tSINT32 PositionX;
	tSINT32 PositionY;

    CMouseMoveEvent(
		tSINT32 PositionX,
		tSINT32 PositionY )
        : CEvent( false ),
		  PositionX( PositionX ),
		  PositionY( PositionY )
	{ };
};

// key events, trivial as classes so this style of compressed writing is hopefully ok...
// actual key hits (presses and released)
class CKeyEscEvent        : public CEvent {  };

//////////////////////////////////////////////

class
CEventHandler
{
public:
    explicit CEventHandler( CEventHandler *nNextHandler );
    virtual ~CEventHandler( );

    virtual void HandleEvent( CEvent *Event );
private:
    CEventHandler *NextHandler;
};

class
CMapEventHandler : public CEventHandler
{
public:
    CMapEventHandler( CBlackFish *BlackFish, CBobbingSprite *MarineMarker, CSprite *CellMarker, CEventHandler *NextHandler );

    void HandleEvent( CEvent *Event );
private:
    CBlackFish        *BlackFish;
    CBobbingSprite    *MarineMarker;

    CSprite            *CellMarker;
    CMarineSprite    *SelectedMarine;
};

class
CMainKeyEventHandler : public CEventHandler
{
public:
    CMainKeyEventHandler( HWND Window, CEventHandler *NextHandler );

    void HandleEvent( CEvent *Event );
private:
	HWND Window;
};
//////////////////////////////////////////////

class
CGame
{
public:
    CGame( HWND WindowHandle, HINSTANCE Instance, bool FullScreen );
    ~CGame( );

	void Run( void );

	void RetrieveFocus( void );

private:
    bool ApplicationActive;
    CSurface    *Primary;
    CSurface    *BackBuffer;
    CSurface    *RenderTarget;

    CTileSet    *Font1;
    CTileSet    *MouseCursor;

    CCache      *Cache;

    CPaint      *Painter;

    CInput      *Input;
    CKeyboard    *KeyBoard;
    CMouse      *Mouse;

    CBlackFish  *BlackFish;

    CMapEventHandler        *MapEventHandler;
    CMainKeyEventHandler    *MainKeyEventHandler;
    CEventHandler         *RootEventHandler;

    std::vector<CMarineSprite *> Marine;
    CBobbingSprite                *TestBobber;

    CSprite                        *Marker;


	// volatile, so the compiler doesn't optimize my "do nothing" loop away

	volatile tSINT64 NextFrameTime;

	const tSINT32	 MaxFPS;


	void ShowDebugInfo( void );

	// copy and assignment is not allowed
    CGame( const CGame &Game );
    CGame & operator = ( const CGame &Game );
};

#endif
