#include "PixelConverter.hpp"

/******************************************************************************
 * BlackFish
 * Copyright (C) 1999-2001 Sren Meyer-Eppler, Jutta Jordans
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/

/******************************************************************************
    History of Changes:
    - 24.07.1999: file created by BuschnicK
******************************************************************************/

#pragma warning ( push, 1 )
#define DIRECTINPUT_VERSION 0x0700
#include <ddraw.h>
#pragma warning ( pop )

#include "general\debug.hpp"
#include "general\helpmacros.hpp"
#include "BlackFish\Surface.hpp"

/*
 * initialize the pixelformat, extract the format from a valid surface
 * which has already been created in this format, i.e. the primary
 */
CPixelConverter::CPixelConverter( CSurface *Surface )
	: RBitCount( 0 ),
	  GBitCount( 0 ),
	  BBitCount( 0 ),
	  RBitPos( 0 ),
	  GBitPos( 0 ),
	  BBitPos( 0 )
{
    ASSERT( Surface != NULL );

    HRESULT ReturnValue = DD_OK;

    DDPIXELFORMAT PixelFormat;
    ZeroMemory( &PixelFormat, sizeof( PixelFormat ) );
    PixelFormat.dwSize = sizeof( PixelFormat );

    IDirectDrawSurface7 *DXSurface = Surface->GetDXSurface();
    ReturnValue = DXSurface->GetPixelFormat( &PixelFormat );
    ASSERT( ReturnValue == DD_OK );

    ASSERT( PixelFormat.dwFlags == DDPF_RGB );
	if ( PixelFormat.dwRGBBitCount != 16 )
	{
		*LOG << "fatal: trying to run in windowed mode with dektop set to wrong bit-depth. value: "
			 << PixelFormat.dwRGBBitCount << std::endl;
		QUIT
	}

    ProcessBits( PixelFormat.dwRBitMask, &RBitPos, &RBitCount );
    ProcessBits( PixelFormat.dwGBitMask, &GBitPos, &GBitCount );
    ProcessBits( PixelFormat.dwBBitMask, &BBitPos, &BBitCount );
}

/*
 * Given three 8bit RGB values compose the 16bit color value
 */
tUINT16
CPixelConverter::RGBToColor( tUINT8 Red, tUINT8 Green, tUINT8 Blue )
{
    tUINT16 RedValue   = static_cast<tUINT16>(( Red   * ( 1 << RBitCount )) >> 8);// = / 256
    tUINT16 GreenValue = static_cast<tUINT16>(( Green * ( 1 << GBitCount )) >> 8);
    tUINT16 BlueValue  = static_cast<tUINT16>(( Blue  * ( 1 << BBitCount )) >> 8);

    RedValue   <<= RBitPos;
    GreenValue <<= GBitPos;
    BlueValue  <<= BBitPos;

    return (static_cast<tUINT16>(RedValue | GreenValue | BlueValue ));
}

/*
 * private Funktion zum ermitteln der Bits-pro-Pixel und Bitpositionen
 * liefert die Anzahl ('NumBits') gesetzter Bits und die Position des ersten
 * signifikanten Bits ('LowBit') in 'Mask'. Mask mu aus Null-Bits und einem
 * zusammenhngenden Block gesetzter Bits bestehen.
 */
void
CPixelConverter::ProcessBits( tUINT32 Mask, tUINT16 *LowBit, tUINT16 *NumBits )
{
    tUINT32 TestMask = 1;

    for ( *LowBit = 0; *LowBit < 32; ( *LowBit )++ )
    {
        if ( Mask & TestMask )
            break;
        TestMask <<= 1;
    }

    TestMask <<= 1;

    for ( *NumBits = 1; *NumBits < 32; ( *NumBits )++ )
    {
        if( !( Mask & TestMask ))
            break;
        TestMask <<= 1;
    }
}
