#ifndef __SPRITE_HPP__
#define __SPRITE_HPP__

/******************************************************************************
 * BlackFish
 * Copyright (C) 1999-2001 Sren Meyer-Eppler, Jutta Jordans
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/

/******************************************************************************
******************************************************************************/

#pragma warning ( push, 1 )
#include <list>
#pragma warning ( pop )

#ifndef __TYPE_HPP__
#include "general\type.hpp"
#endif

#ifndef __VECTOR_HPP__
#include "general\vector.hpp"
#endif

#ifndef __MAP_HPP__
#include "Map\map.hpp"
#endif

#ifndef __OBSERVER_HPP__
#include "general/Observer.hpp"
#endif

/******************************************************************************************/

class
CSprite
{
public:
	enum { FLAGBLOCKING	= 2 << 0 }; // define other Sprite traits here

    CSprite( CMap *Map, tSINT32 TileSet, tSINT32 TileNr, 
        const CVector & Position, tSINT32 DrawingOrder = 0 );
    virtual ~CSprite( );

	virtual tSINT32 GetTileSet( void );
	virtual tSINT32 GetTileNr( void );

    virtual bool SetPosition( const CVector & Position );
    virtual void GetPosition( CVector & Position );

	virtual void Think( void );

	tUINT32 GetFlags( void );

	void	SetDrawingOrder( tSINT32 Priority );
	tSINT32 GetDrawingOrder() const;

protected:
	tSINT32 TileSet;
	tSINT32 TileNr;
    CMap *Map;

    CVector Position;
	tUINT32 Flags;

	// 0 is default (Marines), higher order sprites at the 
	// same Position will be drawn on top of lower order sprites.
	tSINT32 DrawingOrder;

private:
	// copy and assignment is not allowed
    CSprite( const CSprite &Sprite );
    CSprite & operator = ( const CSprite &Sprite );

    friend bool SpriteComparer( const CSprite * lhs, const CSprite * rhs );
};

bool SpriteComparer( const CSprite * lhs, const CSprite * rhs );

/******************************************************************************************/

class
CMarineSprite : public CSprite, public CBroadcaster
{
public:
    CMarineSprite( CMap *Map, tSINT32 TileSet, const CVector & Position, tSINT32 DrawingOrder = 0 );

	virtual tSINT32 GetTileNr( void );

	virtual void Think( void );
    void Move( const CVector & To );

private:
	enum { UnitsPerSecond = 70 };	// movement speed in worldunits/second
	enum { Frequency = 15 };		// animation frames per second

	tSINT64 LastAnimTime;	// time in ticks of last animation frame change
	tSINT64 LastFrameTime;	// time in ticks of previous frame
	tSINT64 FrameTime;		// time in ticks of current frame
	tSINT32 TileNrBase;
	tSINT32 SequenceLength;

	tDIRECTION Direction;
    CVector Speed;

    std::list< CVector > Path;

	// copy and assignment is not allowed
	// this is already obvious from the base class, but MSVC barks with warning 4511, if
	// I don't explicitly make these private again
    CMarineSprite( const CMarineSprite &MarineSprite );
    CMarineSprite & operator = ( const CMarineSprite &MarineSprite );
};

/******************************************************************************************/

class
CBobbingSprite : public CSprite, public CListener
{
public:
    CBobbingSprite( CMap *Map, tSINT32 TileSet, const CVector & Position, CMarineSprite *Marine, tSINT32 DrawingOrder = 16 );

	virtual tSINT32 GetTileNr( void );
	virtual void Think( void );
    virtual void Update( CBroadcaster *Subject );
    void SetSubject( CMarineSprite *Subject );
    void SetVisible( bool Visible );

private:
	enum { UnitsPerSecond = 50 };
	enum { ZMIN = 85, ZMAX = 100 };

    CMarineSprite *Marine;

    bool Visible;

	tSINT64 LastFrameTime;	// time in ticks of previous frame
	tSINT64 FrameTime;		// time in ticks of current frame
    CVector Speed;

	// copy and assignment is not allowed
    CBobbingSprite( const CBobbingSprite &BobbingSprite );
    CBobbingSprite & operator = ( const CBobbingSprite &BobbingSprite );
};

/******************************************************************************************/

#endif
