#ifndef __SURFACE_HPP__
#define __SURFACE_HPP__

/******************************************************************************
 * BlackFish
 * Copyright (C) 1999-2001 Sren Meyer-Eppler, Jutta Jordans
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/

/******************************************************************************
    Purpose:
    provide access to a DirectXSurface, be it in system or in videomemory
******************************************************************************/

#pragma warning ( disable : 4511 4512 )
#pragma warning ( disable : 4786 )
#pragma warning ( push, 1 )
#ifndef __WINDOWS_H__
#define __WINDOWS_H__
  #ifndef WIN32_LEAN_AND_MEAN // !! reset this after including windows.h?
  #define WIN32_LEAN_AND_MEAN
  #endif
#include <windows.h> // !! nur fr RECT ?!
#endif
#include <list>
#include <boost/utility.hpp>
#pragma warning ( pop )

#include "General/Type.hpp"

class CDxDraw;
struct IDirectDrawSurface7;

class
CSurface : private boost::noncopyable
{
public:
    CSurface();
    ~CSurface();

    void Create();
    void Create( CSurface *Primary );
    void Create( tSINT32 Width, tSINT32 Height, bool VideoMemory );

    void Flip();
	void Blit	  ( const CSurface &Source, const RECT *SourceRect, const RECT *DestRect ) const;
    void BlitTrans( const CSurface &Source, const RECT *SourceRect, const RECT *DestRect ) const;

    void	SetTransparentColor( tUINT32 Color ); // !! 16bit Farbtiefe?
    tUINT32 GetTransparentColor() const;

    void RestoreAllIfLost() const;

    void   Lock();
    void UnLock();

    IDirectDrawSurface7 *GetDXSurface()			const	{ return m_Surface;		};
    tUINT16				*GetSurfaceMemPointer() const	{ return m_SurfaceMem;	};
    bool				 IsLocked()				const	{ return m_Locked;		};
    tSINT32				 GetPitch()				const	{ return m_Pitch;		};
    tSINT32				 GetWidth()				const	{ return m_Width;		};
    tSINT32				 GetHeight()			const	{ return m_Height;		};
    RECT				 GetSize()				const;

    void  SetCacheStatus( void *CachePosition ) { m_ImCachedHere = CachePosition; };
    void *GetCacheStatus( void ) const			{ return m_ImCachedHere;		  };

private:
    IDirectDrawSurface7 *	m_Surface;
    tUINT16 *				m_SurfaceMem;
    tSINT32					m_Pitch;
    void *					m_ImCachedHere;
    tSINT32					m_Width;
    tSINT32					m_Height;
    bool					m_Locked;
};

bool Clip( const RECT & ClipRect, RECT & Source, RECT & Dest );

#endif
