/******************************************************************************
 * BlackFish
 * Copyright (C) 1999-2001 Sren Meyer-Eppler, Jutta Jordans
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/

//-------------------------------------------------------------------------------------------------
///    Purpose:
///    Testumgebung fr neue Module
///    everything in here is a quick hack to test the seperate modules
///    -> it should *not* be used as a base for "the real thing"
//-------------------------------------------------------------------------------------------------

#pragma warning ( push, 1 )
#ifndef __WINDOWS_H__
#define __WINDOWS_H__
  #ifndef WIN32_LEAN_AND_MEAN // !! reset this after including windows.h?
  #define WIN32_LEAN_AND_MEAN
  #endif
#include <windows.h>
#endif
#pragma warning ( pop )

#include "general\debug.hpp"
#include "general\helpmacros.hpp"
#include "BlackFish\game.hpp"
#include "resource.h"

// !! Should be read from a resource or so, whatever...
#define NAME    "BuschnicKs"
#define TITLE   "BuschnicKs Test Window"

CGame *Game = NULL;
bool ApplicationActive = true;

#ifdef _DEBUG
const bool FULLSCREEN = false;
#else
const bool FULLSCREEN = true;
#endif

/*
 *
 */
LRESULT CALLBACK
WindowProcedure( HWND WindowHandle, UINT Message, WPARAM wParam, LPARAM lParam )
{
    switch ( Message )
    {
        case WM_ACTIVATEAPP:
            ApplicationActive = (wParam == WA_ACTIVE) || (wParam == WA_CLICKACTIVE);
            if (( ApplicationActive ) && ( Game != NULL )) // Game is NULL on first activate
				Game->RetrieveFocus( );
        return 0L;

        case WM_CLOSE:
			SAFEDELETE( Game );
            PostQuitMessage(0);
        return 0L;

        case WM_SETCURSOR:
            SetCursor( NULL );
        return true;
    }
    return DefWindowProc( WindowHandle, Message, wParam, lParam );
}

/*
 *
 */
void
InitApplication( HINSTANCE hInstance, int nCmdShow )
{
	HWND        WindowHandle;
    WNDCLASS    WindowClass;

    // Set up and register window class
    WindowClass.style            = CS_HREDRAW | CS_VREDRAW;
    WindowClass.lpfnWndProc      = WindowProcedure;
    WindowClass.cbClsExtra       = 0;
    WindowClass.cbWndExtra       = 0;
    WindowClass.hInstance        = hInstance;
    WindowClass.hIcon            = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON1));
    WindowClass.hCursor          = LoadCursor(NULL, IDC_ARROW);
    WindowClass.hbrBackground    = (HBRUSH )GetStockObject(BLACK_BRUSH);
    WindowClass.lpszMenuName     = NAME;
    WindowClass.lpszClassName    = NAME;
    ATOM AtomValue = RegisterClass( &WindowClass );
	ASSERT( AtomValue != 0 );

	RECT Client;
	SetRect( &Client, 64, 64, 800 + 64, 600 + 64 );
	tSINT32 ReturnValue = AdjustWindowRectEx(
		&Client,
		WS_OVERLAPPEDWINDOW,
        false,
		0 );
	ASSERT( ReturnValue != 0 );

    // Now that windows knows about our new type of window create one
    WindowHandle = CreateWindowEx(
        0,//WS_EX_TOPMOST,
        NAME,
        TITLE,
        WS_OVERLAPPEDWINDOW, //WS_POPUP
        Client.left,
        Client.top,
        Client.right - Client.left,
        Client.bottom - Client.top,
        NULL,
        NULL,
        hInstance,
        NULL );

    ASSERT( WindowHandle != NULL );

    ShowWindow( WindowHandle, nCmdShow );
    ReturnValue = UpdateWindow( WindowHandle );
	ASSERT( ReturnValue != 0 );
    SetFocus( WindowHandle );

    Game = new CGame( WindowHandle, hInstance, FULLSCREEN );
}

/*
 *
 */
extern "C" int PASCAL
WinMain(
    HINSTANCE Instance,
    HINSTANCE PreviousInstance,
    LPSTR CommandLine,
    int CommandShow)
{
    InitLogging( "log.txt" );

    // !! Dummies to avoid the 'Not used' warning
    PreviousInstance = PreviousInstance;
    CommandLine      = CommandLine;

    InitApplication( Instance, CommandShow );

    MSG Message;
    for ( ;; )
    {
        if ( PeekMessage( &Message, NULL, 0, 0, PM_NOREMOVE ))
        {
            if ( !GetMessage( &Message, NULL, 0, 0 ))
			{
			    ShutDownLogging();
                return static_cast<int>( Message.wParam );
			}
            TranslateMessage( &Message );
            DispatchMessage( &Message );
        }
        else if ( ApplicationActive )
        {
			Game->Run( );
        }
        else
        {
            tSINT32 ReturnValue = WaitMessage( );
			ASSERT( ReturnValue != 0 );
        }
    }
	// this point is never reached
    ASSERT( false );
}
