#ifndef __TILESET_HPP__
#define __TILESET_HPP__

/******************************************************************************
 * BlackFish
 * Copyright (C) 1999-2001 Sren Meyer-Eppler, Jutta Jordans
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/

/******************************************************************************
    Purpose:
    Loading and managing of an array of CSurfaces
******************************************************************************/

#pragma warning ( disable : 4511 4512 )
#pragma warning ( disable : 4786 )
#pragma warning ( push, 1 )
#ifndef __WINDOWS_H__
#define __WINDOWS_H__
  #ifndef WIN32_LEAN_AND_MEAN // !! reset this after including windows.h?
  #define WIN32_LEAN_AND_MEAN
  #endif
#include <windows.h> // !! nur fr RECT ?!
#endif
#include <boost/utility.hpp>
#pragma warning ( pop )

#include "General/Type.hpp"
#include "BlackFish/Surface.hpp"
#include "BlackFish/Cache.hpp"

class CDxDraw;

class
CTileSet : private boost::noncopyable
{
public:
    CTileSet( const char *IniFileName, const char *ResourceName );
    ~CTileSet();

    CSurface   *GetTile( tSINT32 TileNr ) const;
    tSINT32		GetTileWidth()  const 		{ return m_TileWidth;  };
    tSINT32		GetTileHeight() const		{ return m_TileHeight; };
    RECT		GetTileRect()   const;
	tSINT32		GetNrOfTiles() const		{ return m_NrOfTiles; };
    void		SetCache( CCache *Cache )	{ m_Cache = Cache; };
    CCache *	GetCache() const	        { return m_Cache;  };

    void Blit( 
		tSINT32 TileNr, 
		const RECT *SourceRect, 
		CSurface &Dest, 
		const RECT *DestRect ) const;

    void BlitTrans( 
		tSINT32 TileNr, 
		const RECT *SourceRect, 
		CSurface &Dest, 
		const RECT *DestRect ) const;

private:
    tSINT32		m_TileWidth;
    tSINT32		m_TileHeight;
    tSINT32		m_SurfaceWidth;
    tSINT32		m_SurfaceHeight;
    tSINT32		m_NrOfTiles;
    CSurface **	m_Tile;
    CCache *	m_Cache;
};

#endif
