#include "write.hpp"

/******************************************************************************
 * BlackFish
 * Copyright (C) 1999-2001 Sren Meyer-Eppler, Jutta Jordans
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/


#pragma warning ( push, 1 )
#ifndef __WINDOWS_H__
#define __WINDOWS_H__
  #ifndef WIN32_LEAN_AND_MEAN // !! reset this after including windows.h?
  #define WIN32_LEAN_AND_MEAN
  #endif
#include <windows.h>
#endif
#include <ddraw.h>
#pragma warning ( pop )

#include "general\helpmacros.hpp"
#include "BlackFish\Surface.hpp"
#include "BlackFish\Paint.hpp"
#include "BlackFish\tileset.hpp"

CWrite *Writer = NULL;

/*
 * constructor, object is unusable in this state, a font has to be selected first
 */
CWrite::CWrite( void )
	: Font( NULL ),
	  RenderTarget( NULL ),
	  CharacterWidth( 0 ),
	  CharacterHeight( 0 ),
	  Spacing( 0 )
{
}

/*
 * the font is a CTileSet (see included example font.ini and font bitmap
 * for the layout and organization)
 */
void
CWrite::SetFont( CTileSet *NewFont )
{
    ASSERT( NewFont != NULL );

    Font = NewFont;

    CharacterWidth	= Font->GetTileWidth();
    CharacterHeight = Font->GetTileHeight();
}

/*
 * write 'Text' to the 'Destination' surface, top left text corner is at
 * DestX/DestY
 *
 * Note: Output is unclipped, resulting in undefined behaviour if text is
 * written out of the destination surface's bounds
 */
void
CWrite::Out( tSINT32 DestX, tSINT32 DestY, const char *Text )
{
    ASSERT( Font != NULL );
    ASSERT( RenderTarget != NULL );
    ASSERT( Text != NULL );

    const char *TextPtr = Text;

    RECT DestRect;
	SetRect( &DestRect, DestX, DestY, DestX + CharacterWidth, DestY + CharacterHeight );

    RECT SourceRect;
	SetRect( &SourceRect, 0, 0, CharacterWidth, CharacterHeight );

    while ( *TextPtr != '\0' )
    {
        // the first 32 ASCII characters are unprintable
        Font->BlitTrans( static_cast<tSINT32>( *TextPtr - 32 ), &SourceRect, *RenderTarget, &DestRect );

        TextPtr++;
        DestRect.left += CharacterWidth;
        DestRect.left += Spacing;
        DestRect.right = DestRect.left + CharacterWidth;
    }
}

/*
 * sets the surface text is drawn to, returns the previous drawing target surface, so
 * callers may restore that after they are done
 */
CSurface *
CWrite::SetDrawingTarget( CSurface *Target )
{
	ASSERT( Target != NULL );
    CSurface *ReturnValue = RenderTarget;
	RenderTarget = Target;
	return ReturnValue;
}

