#include "editfish\EditFish.hpp"

/******************************************************************************
 * BlackFish
 * Copyright (C) 1999-2001 Sren Meyer-Eppler, Jutta Jordans
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/

/***********************************************************************************************************
   Notes: Nachkomme von BlackFish mit zustzlicher MenuFunktionalitt

   History of Changes:
   - 9.10.1999 file created by Capella
   - 11.10.1999 first functionality reached, draws floortiles on map
   - 15.10.1999 now also draws walls and overlays, shows complete tile after right click
   - 18.10.1999 map optimizer added
   - 30.10.1999 tiles are now deletable
   - 03.11.1999 save function added, writes map to binary file
   - 17.03.2000 ShowTile updates on left click
   - 19.03.2000 Map functionality now in extra class
   - 24.06.2000 BuschnicK
				search for !!! a delete missing?
				crashes when trying to acces tileset nr 9 -> the one after the character anim (wrap error?)
				modified functions to use the new map interface
				commented out the newmapdialog, since it's nonfunctional at the moment anyways
				8 sign mismatch warnings?
				feature wishlist:
				when modifying a cell in the editor make that the active showtile
				implement 'drawing' = holding down the mouse button to place tiles
				implement delete key to erase walls or tiles
				make an option to just draw on the top layer automatically
				implement a 'place random tile in range x to y'
************************************************************************************************************/

#include "BlackFish\cache.hpp"
#include "BlackFish\tileset.hpp"
#include "BlackFish\write.hpp"
//#include "general\debug.hpp"
//#include "general\HelpMacros.hpp"
//#include "map\Map.hpp"
//#include "BlackFish\surface.hpp"
#include "BlackFish\paint.hpp"
#include "BlackFish\time.hpp"
//#include "BlackFish\dialog.hpp"
#include "BlackFish\input.hpp"
#include "EditFish\Selection.hpp"
#include "EditFish\MiniMap.hpp"
#include "IniFile\IniFile.hpp"

CEditFish::CEditFish(CSurface *nSystemBuffer,
                     CCache *nCache,
					 const char *IniFileName)
         : CBlackFish( nSystemBuffer, nCache, IniFileName)
		 , Buttons( NULL )
		 , MenuTiles( NULL )
		 , ActiveTilesNr( 1 )
		 , ActiveTiles( NULL )
		 , VisibleTile( 0 )
		 , WorkingLayerNr( 0 )
		 , ShowTile( NULL )
		 , ShowTilePosX( 0 )
		 , ShowTilePosY( 0 )
		 , ActiveTilesKind( GROUND )
		 , ActiveNrOfTileSets( 0 )
		 , CurrentIndexOffset( 0 )
		 , PaintSelection( NULL )
         , LeftButtonDown( false )
		 , MouseCellX( 0 )
		 , MouseCellY( 0 )
		 , StartTime( 0 )
		 , MiniMap( NULL )
         , WasInSelection( false )
         , SmallWalls( false )
		 , UndoFile( "temp\\undo" )
		 , UndoIndex( 0 )
		 , UndoLevel( 0 )
		 , AllowedUndos( 0 )
		 , MAXUNDO(10)
		 , MAXLAYER(4)
		 , EraseMode( E_NONE )
{
    // here the Tileset that contains the Menubuttons is defined and loaded
    MenuTiles = new CTileSet( "graphics\\edit.ini", "[Edit1]" );
	ASSERT( MenuTiles != NULL);
    MenuTiles->SetCache ( Cache );

	PaintSelection = new cSELECTION();
	ASSERT( PaintSelection != NULL );

	MiniMap = new cMINIMAP();
	ASSERT( MiniMap != NULL );
	MiniMap->Refresh(Map);

    Buttons = new cBUTTONARRAY("graphics\\edit.ini");
	ASSERT( Buttons != NULL );

    ShowTile = new CWallinfo[6];  //Tile that is selected with right click and shown on bottom of screen
	ASSERT( ShowTile != NULL );

    tUINT32 i;
    for ( i = 0; i < 6; i++ )
    {
        ShowTile[i].TileSet = 0;
        ShowTile[i].TileNr  = 0;
    }

	ActiveNrOfTileSets = GetNrOfTileSets(ActiveTilesKind);
	CurrentIndexOffset = (NrOfTileSets / 4) * ActiveTilesKind;

    ActiveTiles   = Tiles[CurrentIndexOffset + ActiveTilesNr];

	PaintSelection->Select(VisibleTile);
}

CEditFish::~CEditFish()
{
	for (tSINT32 i = 0; i < MAXUNDO; i++)
	{
		//delete the backup files, might be possible to use wild cards here
		std::string CurrentFile = UndoFile;
		char IndexString[3];
		CurrentFile += itoa(i, IndexString, 10);
		CurrentFile += ".tmp";
		DeleteFile(CurrentFile.c_str());
	}

    SAFEDELETE( Buttons );
    SAFEDELETE( MenuTiles );
	SAFEDELETE( PaintSelection );
	SAFEDELETE( MiniMap );
	SAFEDELETEARRAY( ShowTile );
	ActiveTiles = NULL;
}

void
CEditFish::Menu(void)
{
    RECT Source;
    RECT Dest;

    /*
     * Show buttons and edit frames
     */
    Source.left     = 0;
    Source.right    = Source.left + 64;
    Source.top      = 0;
    Source.bottom   = Source.top + 32;

    MenuTiles->Blit( 0, &Source, *RenderTarget, &(Buttons->GetButton(0)->Rect) );
    MenuTiles->Blit( 1, &Source, *RenderTarget, &(Buttons->GetButton(1)->Rect) );
	//OptimizeButton
    MenuTiles->BlitTrans( 7, &Source, *RenderTarget, &(Buttons->GetButton(2)->Rect) );
	//NewButton
	MenuTiles->BlitTrans( 11, &Source, *RenderTarget, &(Buttons->GetButton(7)->Rect) );
	//LoadButton
	MenuTiles->BlitTrans(9, &Source, *RenderTarget, &(Buttons->GetButton(8)->Rect) );
	//SaveButton
	MenuTiles->BlitTrans(10, &Source, *RenderTarget, &(Buttons->GetButton(9)->Rect) );

    /*
     * show left and right buttons
     * !!not very nicely done, since BlitRectangle much bigger than actual button
     */
    Dest.left       = Buttons->GetButton(3)->Rect.left;
    Dest.right      = Dest.left + 64;
    Dest.top        = Buttons->GetButton(3)->Rect.top;
    Dest.bottom     = Dest.top + 32;

    MenuTiles->BlitTrans( 3, &Source, *RenderTarget, &Dest );
    MenuTiles->BlitTrans( 4, &Source, *RenderTarget, &Dest );

    // should write the number of the active tile set
    char String[4];
    ltoa( ActiveTilesNr, String, 10);
	Writer->SetDrawingTarget( RenderTarget );
    Writer->Out( 736, 4, String );

    /*
     * show layer buttons
     */
    Dest.left       = Buttons->GetButton(5)->Rect.left;
    Dest.right      = Dest.left + 64;
    Dest.top        = Buttons->GetButton(5)->Rect.top;
    Dest.bottom     = Dest.top + 32;

    MenuTiles->BlitTrans( 3, &Source, *RenderTarget, &Dest );
    MenuTiles->BlitTrans( 4, &Source, *RenderTarget, &Dest );

    // should write the number of the active layer
    ltoa( WorkingLayerNr, String, 10);
    Writer->Out( 724, 480, "Layer");
    Writer->Out( 736, 500, String );

    tSINT32 Height;
    Height = ActiveTiles->GetTileHeight() ;
    if ( Height == 32 )
    {
        // blits EditFrame for FloorTiles
        MenuTiles->BlitTrans( 2, &Source, *RenderTarget, &(Buttons->GetButton(17)->Rect) );

        tSINT32 i;
        tSINT32 TileCount;
        if ( ActiveTiles->GetNrOfTiles() >= 12)
        {
            for ( i = -6; i <=5 ; i++ )
            {
                TileCount = VisibleTile + i;
                if ( TileCount < 0 )
                {
                    TileCount =  ActiveTiles->GetNrOfTiles() + TileCount;
                }
                else if ( TileCount >= static_cast<tSINT32> (ActiveTiles->GetNrOfTiles()) )
                {
                    TileCount = abs( ActiveTiles->GetNrOfTiles() - TileCount );
                }

                // should blit 12 tiles, 17 is the position of the tile in edit frame (VisibleTile)
                ActiveTiles->BlitTrans( TileCount, &Source, *RenderTarget, &(Buttons->GetButton(17+i)->Rect) );
                Buttons->GetButton(17+i)->Active = true;

				// blit green frame for selected tiles
				if((PaintSelection->IsSelected(TileCount))&&(TileCount != VisibleTile))
				{
					MenuTiles->BlitTrans( 12, &Source, *RenderTarget, &(Buttons->GetButton(17+i)->Rect) );
				}
            }
        }
        else
        {
            for ( i= -6; i <= 5; i++ )
            {
                TileCount = VisibleTile + i;
                if (( TileCount >= 0 ) && (TileCount < ActiveTiles->GetNrOfTiles() ))
                {
                        ActiveTiles->BlitTrans( TileCount,
												&Source,
												*RenderTarget,
												&(Buttons->GetButton(17+i)->Rect) );
                        Buttons->GetButton(17+i)->Active = true;
							// blit green frame for selected tiles
					if((PaintSelection->IsSelected(TileCount))&&(TileCount != VisibleTile))
					{
						MenuTiles->BlitTrans( 12, &Source, *RenderTarget, &(Buttons->GetButton(17+i)->Rect) );
					}
                }
                else
                {
                    if ( TileCount < 0 )
                    {
                        if ( ( 12 + TileCount ) < ActiveTiles->GetNrOfTiles() )
                        {
                            ActiveTiles->BlitTrans( 12 + TileCount,
													&Source,
													*RenderTarget,
													&(Buttons->GetButton(17+i)->Rect) );
                            Buttons->GetButton(17+i)->Active = true;
								// blit green frame for selected tiles
							if(PaintSelection->IsSelected(TileCount + 12))
							{
								MenuTiles->BlitTrans( 12, &Source, *RenderTarget, &(Buttons->GetButton(17+i)->Rect) );
							}
                        }
                        else
                        {
                            Buttons->GetButton(17+i)->Active = false;
                        }
                    }
                    else
                    {
                        if (( TileCount - 12 ) >= 0 )
                        {
                            ActiveTiles->BlitTrans( TileCount - 12,
													&Source,
													*RenderTarget,
													&(Buttons->GetButton(17+i)->Rect));
                            Buttons->GetButton(17+i)->Active = true;
								// blit green frame for selected tiles
							if(PaintSelection->IsSelected(TileCount - 12))
							{
								MenuTiles->BlitTrans( 12, &Source, *RenderTarget, &(Buttons->GetButton(17+i)->Rect) );
							}
                        }
                        else
                        {
                            Buttons->GetButton(17+i)->Active = false;
                        }
                    }
                }
            }
        }
    }
    else
    {
        //show two parts of edit frame for WallTiles
        MenuTiles->BlitTrans( 5, &Source, *RenderTarget, &(Buttons->GetButton(15)->Rect) );
        MenuTiles->BlitTrans( 6, &Source, *RenderTarget, &(Buttons->GetButton(18)->Rect) );

        tSINT32 i;
        tSINT32 TileCount;

        for ( i = -1; i <=1 ; i++ )
        {
            TileCount = VisibleTile + i;
            if ( TileCount < 0 )
            {
				if(ActiveTiles->GetNrOfTiles() > 2)
				{
					TileCount = ActiveTiles->GetNrOfTiles() + TileCount;
				}
				else
				{
					TileCount = 3;
				}
            }
			else if ( TileCount >= ActiveTiles->GetNrOfTiles() )
            {
				if(ActiveTiles->GetNrOfTiles() > 2)
				{
					TileCount = abs( ActiveTiles->GetNrOfTiles() - TileCount );
				}
				else
				{
					TileCount = 3;
				}
            }

            // 15 is the middle of the tile in edit frame (VisibleTile), hence:
            Dest.left    = Buttons->GetButton(17)->Rect.left ;
            Dest.right   = Buttons->GetButton(17)->Rect.right ;
            Dest.top     = Buttons->GetButton(15+(4*i))->Rect.top;
            Dest.bottom  = Buttons->GetButton(18+(4*i))->Rect.bottom ;

            Source.left   = 0 ;
            Source.right  = Source.left + 64;
            Source.top    = 0 ;
            Source.bottom = Source.top + 128;

            // should blit only tiles that really exist
            if(TileCount < ActiveTiles->GetNrOfTiles())
			{
				ActiveTiles->BlitTrans( TileCount, &Source, *RenderTarget, &Dest );
                Buttons->GetButton(15+(4*i))->Active = true;
                Buttons->GetButton(16+(4*i))->Active = true;
                Buttons->GetButton(17+(4*i))->Active = true;
                Buttons->GetButton(18+(4*i))->Active = true;
			}
			else
			{
                Buttons->GetButton(15+(4*i))->Active = false;
                Buttons->GetButton(16+(4*i))->Active = false;
                Buttons->GetButton(17+(4*i))->Active = false;
                Buttons->GetButton(18+(4*i))->Active = false;
			}
	    }
	}

    /*
     * show all layers of a selected showtile
     */
    tUINT32 i;
    tSINT32 ShowX, ShowY;
    ShowX = 100;
    ShowY = 500;

    Source.left   = 0;
    Source.right  = Source.left + 64;
    Source.top    = 0;
    Source.bottom = Source.top + 32;

    for ( i = 0; i <=5; i++)
    {
        Dest.left   = ShowX;
        Dest.right  = Dest.left + 64;
        Dest.top    = ShowY;
        Dest.bottom = Dest.top + 32;

        Tiles[ShowTile[i].TileSet]->BlitTrans( ShowTile[i].TileNr, &Source, *RenderTarget, &Dest );
        ShowX = ShowX +  64;
    }

	// show eraser
	MenuTiles->Blit(13, &Source, *RenderTarget, &(Buttons->GetButton(10)->Rect));
	WriteEraseMode( Buttons->GetButton(10)->Rect.left + 8,
					Buttons->GetButton(10)->Rect.top  + 8);
	if(EraseMode == E_NONE)
	{
		MenuTiles->BlitTrans(14, &Source, *RenderTarget, &(Buttons->GetButton(10)->Rect));
	}

	//blit small walls button
	MenuTiles->Blit(15, &Source, *RenderTarget, &(Buttons->GetButton(30)->Rect));

	// blit undo button
	MenuTiles->Blit(16, &Source, *RenderTarget, &(Buttons->GetButton(31)->Rect));
    if(Buttons->GetButton(31)->Active == false)
	{
		MenuTiles->BlitTrans(14, &Source, *RenderTarget, &(Buttons->GetButton(31)->Rect));
	}

	// blit redo button
	MenuTiles->Blit(17, &Source, *RenderTarget, &(Buttons->GetButton(32)->Rect));
    if(Buttons->GetButton(32)->Active == false)
	{
		MenuTiles->BlitTrans(14, &Source, *RenderTarget, &(Buttons->GetButton(32)->Rect));
	}

    /*
     * blit yellow frame around show tile on map
	 * note that source rect can be clipped after this!!
     */
    GetBlitRectForCell (Dest, ShowTilePosX, ShowTilePosY);

    if ( Clip ( m_ViewRect, Source, Dest ))
    {
        MenuTiles->BlitTrans( 8, &Source, *RenderTarget, &Dest );
    }

	/*
	 *	for debug reasons: show cache dump
	 */
//	DEBUGCODE ( Cache64x128->Dump(RenderTarget, 0)  );

	/*
	 *	show MiniMap
	 */
	MiniMap->Blit(RenderTarget, 0, 500);

	char CellString[20];
	Writer->Out(3, 16, "CellX:");
	Writer->Out(3, 32, "CellY:");
	if((Map) && (Map->IsValidCell(MouseCellX, MouseCellY)))
	{
		ltoa(MouseCellX, CellString, 10);
		Writer->Out(64, 16, CellString);
		ltoa(MouseCellY, CellString, 10);
		Writer->Out(64, 32, CellString);
	}
	else
	{
		Writer->Out(64, 16, "-");
		Writer->Out(64, 32, "-");
	}
}

void
CEditFish::MouseClick( tSINT32 MouseX, tSINT32 MouseY, tUINT32 MouseButtonNr )
{
    /*
     * check if a tile is hit
     */
    if ( ( ( MouseX >= m_ViewRect.left ) && ( MouseX <= m_ViewRect.right) ) &&
         ( ( MouseY >= m_ViewRect.top )  && ( MouseY <= m_ViewRect.bottom ) ) )
    {
        // Variables for ArrayTile that has been hit
        tSINT32 HitCellX    = 0 ;
        tSINT32 HitCellY    = 0 ;

        if(GetCellForScreen(HitCellX, HitCellY, MouseX, MouseY))
		{
	        /*
		     * right click on tile makes it ShowTile
			 */
			if ( MouseButtonNr == 2 )
			{
				UpdateShowTile(HitCellX, HitCellY);
			}
		}
    }

    /*
     * check if a menu button is hit
     */
    else
    {
        for ( tSINT32 i = 0; i < Buttons->GetNrOfButtons(); i++)
        {
			tBUTTON* ThisButton = Buttons->GetButton(i);
            if ((( MouseX >= ThisButton->Rect.left ) && ( MouseX <= ThisButton->Rect.right )) &&
                (( MouseY >= ThisButton->Rect.top ) && ( MouseY <= ThisButton->Rect.bottom )))
            {
                if ( ThisButton->Active )
                {
                   ActivateButton ( i, MouseButtonNr );
                }
            }
        }
    }
}

/*
 *	takes care of dragging moves
 */
void
CEditFish::MouseState(tSINT32 PosX, tSINT32 PosY, tUINT32 MButtons)
{
	// everything over the array area
	 if ((( PosX >= m_ViewRect.left ) && ( PosX <= m_ViewRect.right )) &&
         (( PosY >= m_ViewRect.top ) && ( PosY <= m_ViewRect.bottom )))
	 {
		 tSINT32 LocalMouseCellX = 0;
		 tSINT32 LocalMouseCellY = 0;

         if (MButtons & CMouse::Button1)
		 {
			 if(GetCellForScreen(LocalMouseCellX, LocalMouseCellY, PosX, PosY))
			 {
				 if(!LeftButtonDown)
				 {
					 //make backup of current map state
					 BackUpMap();
                     LeftButtonDown = true;
					 MouseCellX = LocalMouseCellX;
					 MouseCellY = LocalMouseCellY;
					 if(EraseMode == E_NONE)
					 {
						 tSINT32 NewTile = PaintSelection->GetRandomSelection();
						 if (NewTile != -1)
						 {
							 PutNewTile(MouseCellX, MouseCellY, NewTile, CurrentIndexOffset + ActiveTilesNr);
							 StartTime = Timer.GetTime();
						 }
					 }
					 else
					 {
						 EraseTile(MouseCellX, MouseCellY);
					 }
				 }
				 else if ((MouseCellX == LocalMouseCellX) &&
						  (MouseCellY == LocalMouseCellY))
				 {
					 if(EraseMode == E_NONE)
					 {
						 // timed replacement of tile
						 double Time = Timer.GetTime() - StartTime;
						 Time = (Timer.GetFrequency() ? Time/Timer.GetFrequency() : 0.0);
						 if (Time > 0.2)
						 {
				 			tSINT32 NewTile = PaintSelection->GetRandomSelection();
							if (NewTile != -1)
							{
								PutNewTile(MouseCellX, MouseCellY, NewTile, CurrentIndexOffset + ActiveTilesNr);
								StartTime = Timer.GetTime();
							}
						 }
					 }
				 }
				 else
				 {
					 MouseCellX = LocalMouseCellX;
					 MouseCellY = LocalMouseCellY;
					 if(EraseMode == E_NONE)
					 {
						 tSINT32 NewTile = PaintSelection->GetRandomSelection();
						 if (NewTile != -1)
						 {
							 PutNewTile ( MouseCellX, MouseCellY, NewTile, CurrentIndexOffset + ActiveTilesNr);
							 StartTime = Timer.GetTime();
						 }
					 }
					 else
					 {
						 EraseTile(MouseCellX, MouseCellY);
					 }
				 }
			 }
		 }
		 else
		 {
			 MouseCellX = LocalMouseCellX;
			 MouseCellY = LocalMouseCellY;
		 }
	 }
	 else
	 {
		 MouseCellX = -1;
		 MouseCellY = -1;
	 }


     if(!(MButtons & CMouse::Button1))
	 {
         LeftButtonDown = false;
	 }
}

void
CEditFish::HandleKeyStroke(tUINT8 KeyMessage)
{
	switch (KeyMessage)
	{
		case KEY_HOME:
			//SetPosition( 0, 0 );
		break;

		case KEY_LEFT:
			//ActivateButton ( 3, 1);
            ChangeTileSet(false);
		break;

		case KEY_RIGHT:
			//ActivateButton( 4, 1);
            ChangeTileSet(true);
		break;

		case KEY_UP:
			ChangeTile (VisibleTile - 1 );
		break;

		case KEY_DOWN:
			ChangeTile ( VisibleTile + 1 );
		break;

		case KEY_0:
			SetWorkingLayer(0);
		break;

		case KEY_1:
			SetWorkingLayer(1);
		break;

		case KEY_2:
			SetWorkingLayer(2);
		break;

		case KEY_3:
			SetWorkingLayer(3);
		break;

		case KEY_4:
			SetWorkingLayer(4);
		break;

		case KEY_R:
			PaintSelection->ReleaseSelection();
			PaintSelection->Select(VisibleTile);
		break;

		case KEY_U:
			UnDo();
		break;

		case KEY_I:
			ReDo();
		break;
	}

}

/*
 * Activates all MenuButtons after they have been hit with the mouse or called for with shortcut
 */
void
CEditFish::ActivateButton ( tSINT32 ButtonNr, tUINT32 MouseButtonNr )
{
	ASSERT( Map != NULL );
    tSINT32 Height;
    Height = ActiveTiles->GetTileHeight();
    switch ( MouseButtonNr )
    {
        //all left clicks handled here
        case 1 :
            switch ( ButtonNr )
            {
                case 0 :
                    ChangeTile ( VisibleTile - 1 );
                break;

                case 1 :
                    ChangeTile ( VisibleTile + 1 );
                break;

                case 2 :
					BackUpMap();
                    Map->Optimize();
					UpdateShowTile(ShowTilePosX, ShowTilePosY);
                break;

                case 3 :
                    ChangeTileSet(false);
                break;

                case 4 :
                    ChangeTileSet(true);
                break;

                case 5 :
                    if ( WorkingLayerNr > 0 )
                    {
                        WorkingLayerNr = WorkingLayerNr - 1;
                    }
                    else
                    {
                        WorkingLayerNr = MAXLAYER;
                    }
                break;

                case 6 :
                    if ( WorkingLayerNr < MAXLAYER )
                    {
                        WorkingLayerNr = WorkingLayerNr + 1;
                    }
                    else
                    {
                        WorkingLayerNr = 0;
                    }
                break;

				//NewButton
				case 7:
					if (Dialog == NULL)
                        Dialog = new CNewMapDialog();
				break;

				//LoadButton
				case 8:
					if (Dialog == NULL)
                        Dialog = new CLoadMapDialog();
				break;

				//SaveButton
				case 9:
					if (Dialog == NULL)
                        Dialog = new CSaveMapDialog(Map->GetName(), Map->GetAuthor(), Map->GetFileName());
					ASSERT( Dialog != NULL);
				break;

				// switch eraser
				case 10:
                    ChangeEraseMode(true);
				break;

                case 11 :
                case 12 :
                case 13 :
                case 14 :
                    if ( Height == 32)
                    {
                        ChangeTile( VisibleTile + ButtonNr - 17 ) ;
                    }
                    else
                    {
                        ChangeTile( VisibleTile - 1 );
                    }
                break;

                case 15 :
                case 16 :
                case 17 :
                case 18 :
                    if ( Height == 32)
                    {
                        ChangeTile( VisibleTile + ButtonNr - 17 ) ;
                    }
                break;

                case 19 :
                case 20 :
                case 21 :
                case 22 :
                    if ( Height == 32)
                    {
                        ChangeTile( VisibleTile + ButtonNr - 17 ) ;
                    }
                    else
                    {
                        ChangeTile( VisibleTile + 1 );
                    }
                break;

                // click in ShowTile changes the ActiveTile
                case 23 :
                case 24 :
                case 25 :
                case 26 :
                case 27 :
                case 28 :
					{
						tSINT32 NewTileSet = ShowTile[ButtonNr - 23].TileSet;
						switch (NewTileSet / 256)
						{
							case 0 :
								ActiveTilesKind = GROUND;
							break;
							case 1 :
								ActiveTilesKind = OVERLAY;
							break;
							case 2 :
								ActiveTilesKind = WALL;
							break;
							case 3 :
								ActiveTilesKind = SPRITE;
							break;
						}
						ActiveNrOfTileSets = GetNrOfTileSets(ActiveTilesKind);
						CurrentIndexOffset = (GetNrOfTileSets(MAX) / 4) * ActiveTilesKind;
						ActiveTiles = Tiles[NewTileSet];
						PaintSelection->ReleaseSelection();
						PaintSelection->Select(VisibleTile);
						ChangeTile(ShowTile[ButtonNr - 23].TileNr);
					}
                break;
				case 29 :
					ChangeKind();
				break;
				case 30 :
					if(SmallWalls)
                        SmallWalls = false;
					else
                        SmallWalls = true;
				break;
				case 31 :
					UnDo();
				break;
				case 32 :
					ReDo();
				break;
            }
         break;

		 //all right clicks handled here
         case 2 :

            switch (ButtonNr)
            {
				//switch erase mode
				case 10:
                    ChangeEraseMode(false);
				break;

				// for all visible tiles, check if one is to be selected
				// or deselected
				case 11 :
				case 12 :
				case 13 :
				case 14 :
					tSINT32 ClickedTile;
					if ( Height == 32)
                    {
                        ClickedTile = VisibleTile + ButtonNr - 17;
                    }
                    else
                    {
						ClickedTile = VisibleTile - 1;
					}

					if (ClickedTile >= 0)
					{
						ClickedTile = ClickedTile % ActiveTiles->GetNrOfTiles();
					}
					else
					{
						tSINT32 NrOfTiles = ActiveTiles->GetNrOfTiles();
						ClickedTile = __max(12, NrOfTiles) - (abs(ClickedTile) % NrOfTiles);
					}

					if (PaintSelection->IsSelected(ClickedTile))
					{
						PaintSelection->Deselect(ClickedTile);
					}
					else
					{
						PaintSelection->Select(ClickedTile);
					}
				break;
				case 15 :
				case 16 :
//				case 17 :
				case 18 :
					if ( Height == 32 )
                    {
                        ClickedTile = VisibleTile + ButtonNr - 17;
						if (ClickedTile >= 0)
						{
							ClickedTile = ClickedTile % ActiveTiles->GetNrOfTiles();
						}
						else
						{
							tSINT32 NrOfTiles = ActiveTiles->GetNrOfTiles();
							ClickedTile = __max(12, NrOfTiles) - (abs(ClickedTile) % NrOfTiles);
						}

						if (PaintSelection->IsSelected(ClickedTile))
						{
							PaintSelection->Deselect(ClickedTile);
						}
						else
						{
							PaintSelection->Select(ClickedTile);
						}
                    }

				break;
				case 19 :
				case 20 :
				case 21 :
				case 22 :
					if ( Height == 32)
                    {
                        ClickedTile = VisibleTile + ButtonNr - 17;
                    }
                    else
                    {
						ClickedTile = VisibleTile + 1;
					}

					if (ClickedTile >= 0)
					{
						ClickedTile = ClickedTile % ActiveTiles->GetNrOfTiles();
					}
					else
					{
						tSINT32 NrOfTiles = ActiveTiles->GetNrOfTiles();
						ClickedTile = __max(12, NrOfTiles) - (abs(ClickedTile) % NrOfTiles);
					}

					if (PaintSelection->IsSelected(ClickedTile))
						 {
							 PaintSelection->Deselect(ClickedTile);
						 }
						 else
						 {
							 PaintSelection->Select(ClickedTile);
						 }
				break;

                // fill layer with empty tile
                case 23 :
                    DeleteShowTileLayer(0);
                break;
                case 24 :
                    DeleteShowTileLayer(1);
                break;
                case 25 :
                    DeleteShowTileLayer(2);
                break;
                case 26 :
                    DeleteShowTileLayer(3);
                break;
                case 27 :
                    DeleteShowTileLayer(4);
                break;
                // delete wall
                case 28 :
					DeleteWall(ShowTilePosX, ShowTilePosY);
                    UpdateShowTile(ShowTilePosX, ShowTilePosY);
                break;
				case 29 :
                    ChangeKind(false);
				break;
            }
         break;

         case 3 :
         break;
    }
}


void
CEditFish::PutNewTile ( tSINT32 NewTilePosX, tSINT32 NewTilePosY, tSINT32 NewTileNr, tSINT32 NewTileSetNr )
{
	ASSERT( Map != NULL );
    // security check, do not write outside of array
    if ( Map->IsValidCell(NewTilePosX, NewTilePosY) )
    {
        tSINT32 Height;
        Height = Tiles[NewTileSetNr]->GetTileHeight();
        if ( Height == 32)
        {
            CCellinfo *Layer;
            Layer = GetWorkingLayer ( NewTilePosX, NewTilePosY );
            Layer->TileSet = NewTileSetNr;
            Layer->TileNr = NewTileNr ;
			if(WorkingLayerNr == 0)
			{
				// ground has been changed, update MiniMap
				MiniMap->RefreshPixel(Map, NewTilePosX, NewTilePosY);

			}
        }
        else
        {
            if ( Map->GetWall( NewTilePosX, NewTilePosY ) == NULL )
            {
                Map->SetWall( NewTilePosX, NewTilePosY, NewTileSetNr, NewTileNr );
				//Map->SetFlags( NewTilePosX, NewTilePosY, FLAGBLOCKING );
            }
			else
			{
				Map->GetWall( NewTilePosX, NewTilePosY )->TileNr = NewTileNr;
				Map->GetWall( NewTilePosX, NewTilePosY )->TileSet = NewTileSetNr;
			}
			// change Minimap
			MiniMap->RefreshPixel(Map, NewTilePosX, NewTilePosY);
        }

		//update ShowTile if neccessary
		if((NewTilePosX == ShowTilePosX) && (NewTilePosY == ShowTilePosY))
		{
			UpdateShowTile(ShowTilePosX, ShowTilePosY);
		}
    }
}

/*
 *	forces the new layer to exist
 */
CCellinfo *
CEditFish::GetWorkingLayer ( tSINT32 NewTilePosX, tSINT32 NewTilePosY)
{
	ASSERT( Map != NULL);
    tSINT32 i, j;
    tSINT32 Counter = -1;
    CCellinfo *Layer = Map->GetFirstLayer(NewTilePosX, NewTilePosY);
    while ( Layer != NULL )
    {
        Counter++;
        Layer = Layer->NextLayer;
    }
    if ( WorkingLayerNr <= ( Counter ))
    {
        Layer = Map->GetFirstLayer(NewTilePosX, NewTilePosY);
        for ( i=0; i < WorkingLayerNr; i++)
        {
            Layer = Layer->NextLayer;
        }
    }
    else
    {
        Layer = Map->GetFirstLayer(NewTilePosX, NewTilePosY);
        for ( i = 0; i < Counter ; i++ )
        {
            Layer = Layer->NextLayer;
        }
        for ( j = Counter; j < WorkingLayerNr; j++ )
        {
            Layer->NextLayer = new CCellinfo();
            Layer = Layer->NextLayer;

            Layer->NextLayer    = NULL;
            Layer->TileNr       = 2;
            Layer->TileSet      = 0;
        }
    }
    return Layer;
}

/*
 *	like GetWorkingLayer, but returns NULL if layer does not exist
 */
CCellinfo*
CEditFish::GetLayer(tSINT32 NewTilePosX, tSINT32 NewTilePosY, tSINT32 LayerNr)
{
	ASSERT( Map != NULL );
	tSINT32 Counter = 0;
    CCellinfo *Layer = Map->GetFirstLayer(NewTilePosX, NewTilePosY);
    while (( Layer != NULL ) && (Counter < LayerNr))
    {
        Counter++;
        Layer = Layer->NextLayer;
    }

	return Layer;
}

void
CEditFish::NewMapHeader(CDialog::cRETURNDATABASE* NewMapData)
{
    CNewMapDialog::cRETURNDATA* MapData =
        dynamic_cast<CNewMapDialog::cRETURNDATA*> ( NewMapData );

	if (MapData)
	{
        CMap::tMAPHEADER MapHeader;

		memcpy(MapHeader.Name,MapData->Name.c_str(), 64);
		memcpy(MapHeader.Author, MapData->Author.c_str(), 64);
		MapHeader.Width   = MapData->Width;
		MapHeader.Height  = MapData->Height;
        MapHeader.Version = CMap::DATAVERSION;
		// make new Map from Header
		NewMap ( MapHeader);
		MiniMap->Refresh(Map);
	}

	//now that everything has been copied, destroy Dialog
	SAFEDELETE (Dialog);
}

void
CEditFish::LoadMap(CDialog::cRETURNDATABASE* Data)
{
    CLoadMapDialog::cRETURNDATA* ReturnData =
        dynamic_cast<CLoadMapDialog::cRETURNDATA*> (Data);

	if (ReturnData)
	{
		ChangeMap( ReturnData->FileName.c_str( ));
		MiniMap->Refresh(Map);
	}

	SAFEDELETE(Dialog);
};

void
CEditFish::SaveMap(CDialog::cRETURNDATABASE* Data)
{
	ASSERT( Map != NULL );
    CSaveMapDialog::cRETURNDATA* ReturnData =
        dynamic_cast<CSaveMapDialog::cRETURNDATA*> (Data);

	if ( ReturnData )
	{
		Map->SetName(ReturnData->MapName.c_str());
		Map->SetAuthor(ReturnData->Author.c_str());
		Map->Save(ReturnData->FileName.c_str());
	}
	SAFEDELETE(Dialog);
}

/*
 *	toggle between different kinds of tiles (ground, overlay, wall, sprite)
 */
void
CEditFish::ChangeKind(bool Up)
{
	if(Up)
	{
		if (ActiveTilesKind == SPRITE)
		{
			ActiveTilesKind = GROUND;
		}
		else
		{
			ActiveTilesKind = static_cast<tTILESETTYPE>
				(static_cast<tSINT32> (ActiveTilesKind) + 1);
		}
	}
	else
	{
		if (ActiveTilesKind == GROUND)
		{
			ActiveTilesKind = SPRITE;
		}
		else
		{
			ActiveTilesKind = static_cast<tTILESETTYPE>
				(static_cast<tSINT32> (ActiveTilesKind) - 1);
		}
	}

	ActiveTilesNr = 1;
	ChangeTile(0);
	ActiveNrOfTileSets = GetNrOfTileSets(ActiveTilesKind);
	CurrentIndexOffset = (GetNrOfTileSets(MAX) / 4) * ActiveTilesKind;
	ActiveTiles = Tiles[CurrentIndexOffset + ActiveTilesNr];
	PaintSelection->ReleaseSelection();
	PaintSelection->Select(VisibleTile);
}

/*
 *	switch to next/prev tile set of same kind
 */
void
CEditFish::ChangeTileSet(bool Up)
{
	if (Up)
	{
		if ( ActiveTilesNr < ( ActiveNrOfTileSets - 1 ))
        {
            ActiveTilesNr = ActiveTilesNr + 1;
            ActiveTiles = Tiles[CurrentIndexOffset + ActiveTilesNr];
            ChangeTile(0);
        }
        else
        {
            ActiveTilesNr = 1;
			ActiveTiles = Tiles[CurrentIndexOffset + ActiveTilesNr];
            ChangeTile(0);
        }
	}
	else
	{
		if ( ActiveTilesNr > 1 )
        {
            ActiveTilesNr = ActiveTilesNr - 1;
            ActiveTiles = Tiles[CurrentIndexOffset + ActiveTilesNr];
            ChangeTile(0);
        }
        else
        {
            ActiveTilesNr = ActiveNrOfTileSets - 1 ;
            ActiveTiles = Tiles[CurrentIndexOffset + ActiveTilesNr];
            ChangeTile(0);
        }
	}
	PaintSelection->ReleaseSelection();
	PaintSelection->Select(VisibleTile);
}

void
CEditFish::ChangeTile(tSINT32 TileNr)
{
	if(!WasInSelection)
		PaintSelection->Deselect(VisibleTile);

	if(TileNr >= 0)
	{
		VisibleTile = TileNr % ActiveTiles->GetNrOfTiles();
	}
	else
	{
		if(ActiveTilesKind == WALL)
		{
			VisibleTile = ActiveTiles->GetNrOfTiles() - (abs(TileNr) % ActiveTiles->GetNrOfTiles());
		}
		else
		{
			VisibleTile = ActiveTiles->GetNrOfTiles() - (abs(TileNr) % ActiveTiles->GetNrOfTiles());
		}
	}

	if(PaintSelection->IsSelected(VisibleTile))
	{
        WasInSelection = true;
	}
	else
	{
        WasInSelection = false;
		PaintSelection->Select(VisibleTile);
	}
}

void
CEditFish::DeleteWall(tSINT32 PosX, tSINT32 PosY)
{
	ASSERT( Map != NULL );
	Map->DeleteWall( PosX, PosY );
	MiniMap->RefreshPixel(Map, PosX, PosY);
}

void
CEditFish::DeleteShowTileLayer(tSINT32 LayerNr)
{
	tSINT32 BackUpLayerNr;
	BackUpLayerNr = WorkingLayerNr;
    WorkingLayerNr = LayerNr;
	// Blankotile has to be 0 if it is a base tile (LayerNr = 0),
	// 2 otherwise
	tSINT32 Blanko = (LayerNr ? 2 : 0);
    PutNewTile ( ShowTilePosX, ShowTilePosY, Blanko, 0 );
    WorkingLayerNr = BackUpLayerNr;
    UpdateShowTile(ShowTilePosX, ShowTilePosY);
}

/*
 *	sets ActiveTilesNr, ActiveTilesKind and CurrentIndexOffset
 *  for an absolute TileSetNr
 */
void
CEditFish::SetTileSet(tSINT32 TileSetNr)
{
	ActiveTilesNr = TileSetNr % (GetNrOfTileSets(MAX) / 4);
	ActiveTilesKind = static_cast<tTILESETTYPE> (TileSetNr / (GetNrOfTileSets(MAX) / 4));
	CurrentIndexOffset = (GetNrOfTileSets(MAX)/4) * static_cast<tUINT32> (ActiveTilesKind);
	ActiveTiles = Tiles[TileSetNr];
}

void
CEditFish::WriteEraseMode(tSINT32 ScreenX, tSINT32 ScreenY)
{
	Writer->SetDrawingTarget(RenderTarget);
	switch (EraseMode)
	{
		case E_ALL:
			Writer->Out(ScreenX, ScreenY, " All");
		break;
		case E_LAYER:
			Writer->Out(ScreenX, ScreenY, "Layer");
		break;
		case E_WALL:
			Writer->Out(ScreenX, ScreenY, " Wall");
		break;
	}
}

/*
 *	switches through the different erase modi in both directions
 */
void
CEditFish::ChangeEraseMode(bool Up)
{
	if(Up)
	{
		switch (EraseMode)
		{
			case E_NONE:
				EraseMode = E_ALL;
			break;
			case E_ALL:
				EraseMode = E_LAYER;
			break;
			case E_LAYER:
				EraseMode = E_WALL;
			break;
			case E_WALL:
				EraseMode = E_NONE;
			break;
		}
	}
	else
	{
		switch (EraseMode)
		{
			case E_NONE:
				EraseMode = E_WALL;
			break;
			case E_ALL:
				EraseMode = E_NONE;
			break;
			case E_LAYER:
				EraseMode = E_ALL;
			break;
			case E_WALL:
				EraseMode = E_LAYER;
			break;
		}
	}
}

void
CEditFish::EraseTile(tSINT32 CellX, tSINT32 CellY)
{
	ASSERT( Map != NULL );
	switch(EraseMode)
	{
		case E_ALL:
			{
				DeleteWall(CellX, CellY);
				Map->DeleteAllLayers(CellX, CellY);
				tSINT32 Backup = WorkingLayerNr;
				WorkingLayerNr = 0;
				PutNewTile(CellX, CellY, 0, 0);
				WorkingLayerNr = Backup;
				MiniMap->RefreshPixel(Map, CellX, CellY);
			}
		break;
		case E_LAYER:
			if(GetLayer(CellX, CellY, WorkingLayerNr))
			{
				// Blankotile has to be 0 if it is a base tile (LayerNr = 0),
				// 2 otherwise
				tSINT32 Blanko = (WorkingLayerNr ? 2 : 0);
				PutNewTile(CellX, CellY, Blanko, 0);
				MiniMap->RefreshPixel(Map, CellX, CellY);
			}
		break;
		case E_WALL:
			DeleteWall(CellX, CellY);
		break;
	}
}

CEditFish::cBUTTONARRAY::cBUTTONARRAY(const char* IniFileName)
	: Buttons( NULL )
	, NrOfButtons( 0 )
{
	Initialize(IniFileName);
}

CEditFish::cBUTTONARRAY::~cBUTTONARRAY()
{
	if(Buttons)
	{
		for( tSINT32 i = 0; i < NrOfButtons; i++ )
		{
			SAFEDELETE(Buttons[i]);
		}
	}
	SAFEDELETEARRAY( Buttons );
}

void
CEditFish::cBUTTONARRAY::Initialize(const char* IniFileName)
{
    CIniFile* IniFile = new CIniFile(IniFileName);

	IniFile->ReadNumber("[Buttonarray]", "NrOfButtons", &NrOfButtons);
	Buttons = new tBUTTON*[static_cast<tUINT32> (NrOfButtons)];
	ASSERT( Buttons != NULL);

	long Temp;
	char NameString[16];
	for (tSINT32 i = 0; i < NrOfButtons; i++)
	{
		Buttons[i] = new tBUTTON;
		sprintf( NameString, "[Button%i]", i);
		IniFile->ReadNumber ( NameString, "Left", &(Buttons[i]->Rect.left) );
		IniFile->ReadNumber ( NameString, "Right", &(Buttons[i]->Rect.right) );
		IniFile->ReadNumber ( NameString, "Top", &(Buttons[i]->Rect.top) );
		IniFile->ReadNumber ( NameString, "Bottom", &(Buttons[i]->Rect.bottom) );
		IniFile->ReadNumber ( NameString, "Active", &Temp );
		Buttons[i]->Active = ( Temp != 0 );
	}

	SAFEDELETE(IniFile);
}

void
CEditFish::UpdateShowTile(tSINT32 PosX, tSINT32 PosY)
{
	ASSERT( Map != NULL );
	// security check, do not write outside of array
    if ( ( ( PosX >= 0 ) && ( PosX < Map->GetWidth() ) ) &&
        ( ( PosY >= 0 ) && ( PosY < Map->GetHeight()) ) )
    {
        // "empty" showtile
        tSINT32 i;
        for ( i = 0; i <= 5; i++ )
        {
            ShowTile[static_cast<tUINT32> (i)].TileSet = 0;
            ShowTile[static_cast<tUINT32> (i)].TileNr  = 0;
        }

        ShowTilePosX = PosX;
        ShowTilePosY = PosY;

        tSINT32 Counter = 0;
        CCellinfo *ThisCell= Map->GetFirstLayer( PosX, PosY );

        ShowTile[static_cast<tUINT32> (Counter)].TileSet = ThisCell->TileSet;
        ShowTile[static_cast<tUINT32> (Counter)].TileNr  = ThisCell->TileNr;

        if ( Map->GetWall( PosX, PosY ) != NULL )
        {
            ShowTile[5].TileSet = Map->GetWall( PosX, PosY )->TileSet;
            ShowTile[5].TileNr = Map->GetWall( PosX, PosY )->TileNr;
            Buttons->GetButton(28)->Active = true;
        }

        while ( ThisCell->NextLayer != NULL )
        {
            ThisCell   = ThisCell->NextLayer;
            Counter++;
            ShowTile[static_cast<tUINT32> (Counter)].TileSet = ThisCell->TileSet;
            ShowTile[static_cast<tUINT32> (Counter)].TileNr = ThisCell->TileNr;
        }

        //enable all buttons which are not empty
        for ( i = 0; i <= Counter; i++ )
        {
			// assumes that all tiles in Blankfloor are invalid
            if (!( ShowTile[static_cast<tUINT32> (i)].TileSet == 0 ))
            {
                Buttons->GetButton(23+i)->Active = true;
            }
			else
			{
                Buttons->GetButton(23+i)->Active = false;
			}
        }
    }
}

void
CEditFish::BackUpMap()
{
	ASSERT( Map != NULL );
	std::string CurrentFile = UndoFile;
	char IndexString[3];
	CurrentFile += itoa(UndoIndex % 10, IndexString, 10);
	CurrentFile += ".tmp";
	Map->Save(CurrentFile.c_str());
	UndoIndex++ ;
	AllowedUndos++;
	AllowedUndos = __min(MAXUNDO - 1, AllowedUndos);
	UndoLevel = 0;
    Buttons->GetButton(31)->Active = true;
    Buttons->GetButton(32)->Active = false;
}

void
CEditFish::UnDo()
{
	ASSERT( Map != NULL );
	if(AllowedUndos > 0)
	{
		if(UndoLevel < AllowedUndos)
		{
            CVector CurrentPos = WorldPos;
			// save current state for possible redo
			std::string CurrentFile = UndoFile;
			char IndexString[3];
			CurrentFile += itoa((UndoIndex) % MAXUNDO, IndexString, 10);
			CurrentFile += ".tmp";
			Map->Save(CurrentFile.c_str());
			// restore last state
			UndoIndex--;
			CurrentFile = UndoFile;
			CurrentFile += itoa(UndoIndex % MAXUNDO, IndexString, 10);
			CurrentFile += ".tmp";
			ChangeMap(CurrentFile.c_str());
			MiniMap->Refresh(Map);
			SetPosition(&CurrentPos);
			UndoLevel++;
            Buttons->GetButton(32)->Active = true;

			if(UndoLevel == AllowedUndos)
			{
                Buttons->GetButton(31)->Active = false;
				//UndoLevel = 0;
				//AllowedUndos = 0;
			}
		}
	}
}

void
CEditFish::ReDo()
{
	if(UndoLevel > 0)
	{
		UndoIndex++;
        CVector CurrentPos = WorldPos;
		std::string CurrentFile = UndoFile;
		char IndexString[3];
		CurrentFile += itoa(UndoIndex % MAXUNDO, IndexString, 10);
		CurrentFile += ".tmp";
		ChangeMap(CurrentFile.c_str());
		MiniMap->Refresh(Map);
		SetPosition(&CurrentPos);
		//AllowedUndos++;
		//AllowedUndos = __min(MAXUNDO - 1, AllowedUndos);
		UndoLevel--;
        Buttons->GetButton(31)->Active = true;
		if(UndoLevel == 0)
		{
            Buttons->GetButton(32)->Active = false;
		}
	}
}

