#ifndef __EDITFISH_HPP__
#define __EDITFISH_HPP__

/******************************************************************************
 * BlackFish
 * Copyright (C) 1999-2001 Sren Meyer-Eppler, Jutta Jordans
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/

/**************************************************************************
  History of Changes:
  - erstellt am 9.10.1999 von Capella
  - 11.10.1999 first functionality reached
 *************************************************************************/

#include "BlackFish\Blackfish.hpp"
#include "BlackFish\Dialog.hpp"

// to prevent includes
class CCache;
class CTileSet;
class CWrite;
class CWallinfo;
class CCellinfo;
class cSELECTION;
class cMINIMAP;


class CEditFish : public CBlackFish
{
private:
	struct tBUTTON
	{
		RECT	Rect;
        bool    Active;
	};

	class cBUTTONARRAY
	{
	private:
		tBUTTON**	Buttons;
		tSINT32		NrOfButtons;
		void Initialize(const char* IniFileName);
	public:
		cBUTTONARRAY(const char* IniFileName);
		~cBUTTONARRAY();
		tBUTTON* GetButton(tSINT32 Number){return Buttons[Number];};
		tSINT32 GetNrOfButtons() {return NrOfButtons;};
	};

    cBUTTONARRAY *Buttons;
    CTileSet    *MenuTiles;
    tSINT32		 ActiveTilesNr;
    CTileSet    *ActiveTiles;
    tSINT32	 	 VisibleTile;
    tSINT32		 WorkingLayerNr;
    CWallinfo    *ShowTile;
    tSINT32		 ShowTilePosX, ShowTilePosY;
	tTILESETTYPE ActiveTilesKind;
	tSINT32		 ActiveNrOfTileSets;
	tSINT32		 CurrentIndexOffset; // is 0, 256, 512 or 768
	cSELECTION	*PaintSelection;
    bool        LeftButtonDown;
	tSINT32		 MouseCellX, MouseCellY;
	tDOUBLE		 StartTime;
	cMINIMAP*	 MiniMap;
    bool         WasInSelection;
    bool         SmallWalls;
    //bool       BackUp;
	std::string	 UndoFile;
	tSINT32		 UndoIndex;
	tSINT32		 UndoLevel;
	tSINT32		 AllowedUndos;
	const tSINT32 MAXUNDO;
	const tSINT32 MAXLAYER;

	enum {E_NONE, E_ALL, E_LAYER, E_WALL} EraseMode;
    //CMap      *Map;


    void PutNewTile ( tSINT32 NewTilePosX, tSINT32 NewTilePosY, tSINT32 NewTileNr, tSINT32 NewTileSetNr );
    CCellinfo *GetWorkingLayer ( tSINT32 NewTilePosX, tSINT32 NewTilePosY );
    CCellinfo *GetLayer(tSINT32 NewTilePosX, tSINT32 NewTilePosY, tSINT32 LayerNr);
	//void GetLoadName(char* LoadName);
    void ChangeKind(bool Up = true);
    void ChangeTileSet(bool Up);
	void ChangeTile(tSINT32 TileNr);
	void DeleteWall(tSINT32 PosX, tSINT32 PosY);
	void DeleteShowTileLayer(tSINT32 LayerNr);
	void SetTileSet(tSINT32 TileSetNr);
	void WriteEraseMode(tSINT32 ScreenX, tSINT32 ScreenY);
    void ChangeEraseMode(bool Up);
	void EraseTile(tSINT32 CellX, tSINT32 CellY);
	void UpdateShowTile(tSINT32 PosX, tSINT32 PosY);
	void BackUpMap();
	void UnDo();
	void ReDo();

	// copy and assignment is not allowed
    // the parent class CBlackFish already permits this, but it must still be here to avoid MSVC warning
    CEditFish( const CEditFish &EditFish );
    CEditFish & operator = ( const CEditFish &EditFish );

    public:
    CEditFish(CSurface *nSystemBuffer,
              CCache *nCache,
              const char *IniFileName);

    ~CEditFish();

    enum { ScreenWidth = 799, ScreenHeight = 599, ScreenCenterX = 399, ScreenCenterY = 239};
    enum { DataVersion = 1 };
    void Menu(void);
    void MouseClick(tSINT32 MouseX, tSINT32 MouseY, tUINT32 MouseButtonNr);
    void ActivateButton (tSINT32 ButtonNr, tUINT32 MouseButtonNr);
    void SetWorkingLayer(tSINT32 nWorkingLayerNr) {WorkingLayerNr=nWorkingLayerNr;};
	void HandleKeyStroke(tUINT8 KeyMessage);
    void NewMapHeader (CDialog::cRETURNDATABASE* NewMapData);
    void LoadMap(CDialog::cRETURNDATABASE* Data);
    void SaveMap(CDialog::cRETURNDATABASE* Data);
	void MouseState(tSINT32 PosX, tSINT32 PosY, tUINT32 Buttons);
    bool GetWallState() {return SmallWalls;};
};

#endif
