
/******************************************************************************
 * BlackFish
 * Copyright (C) 1999-2001 Sren Meyer-Eppler, Jutta Jordans
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/

/******************************************************************************
    Purpose:
    Testumgebung fr Editor

    Notes:

    History of Changes:
    - 09.10.1999: file created by Capella (mainly stolen from BuschnicK)
	- 17.03.2000: project now compiles on VisualC++ 6.0

******************************************************************************/

#pragma warning ( push, 1 )
#ifndef __WINDOWS_H__
#define __WINDOWS_H__
  #ifndef WIN32_LEAN_AND_MEAN // !! reset this after including windows.h?
  #define WIN32_LEAN_AND_MEAN
  #endif
#include <windows.h>
#endif
#include <stdlib.h>                     // rand, itoa
#include <ddraw.h>                      // Clipper
#include <mmsystem.h>                   // timeGetTime()
#pragma warning ( pop )

#include "general\debug.hpp"
#include "general\helpmacros.hpp"
//#include "general\type.hpp"
#include "BlackFish\Blackfish.hpp"
#include "BlackFish\Cache.hpp"
#include "BlackFish\DXDraw.hpp"
#include "BlackFish\Input.hpp"
#include "BlackFish\Surface.hpp"
#include "BlackFish\PixelConverter.hpp"
#include "BlackFish\paint.hpp"
#include "BlackFish\TileSet.hpp"
#include "BlackFish\write.hpp"
#include "BlackFish\Dialog.hpp"
#include "BlackFish\Time.hpp"
#include "EditFish\EditFish.hpp"
#include "resource.h"

// !! Should be read from a resource or so, whatever...
#define NAME        "Editor"
#define TITLE       "BlackFish Map Editor"
#define WHEELUP     120
#define WHEELDOWN   -120

#ifdef _DEBUG
const bool FullScreen = false;
#else
const bool FullScreen = true;
#endif

bool ApplicationActive         = false;

CSurface    *Primary            = NULL;
CSurface    *BackBuffer         = NULL;


CTileSet    *Font1              = NULL;
CTileSet    *MouseCursor        = NULL;
CCache      *Cache                = NULL;
CPaint      *Painter            = NULL;

CInput      *DXInput            = NULL;
CMouse        *Mouse              = NULL;
CKeyboard    *KeyBoard           = NULL;

CEditFish   *EditFish           = NULL;


// global for the posting of messages in WinMain
HWND        WindowHandle;

tSINT32 MouseX = 0, MouseY = 0, WheelPos = 0;
tUINT32 Button = 0;
bool  Click1 = false, Click2 = false, Click3 = false;
tSINT32 ScrollRate = 1;


void
ShowMiscInfo( void )
{
}


/*
 *
 */
long FAR PASCAL
WindowProcedure( HWND WindowHandle, UINT Message, WPARAM wParam, LPARAM lParam )
{
    switch ( Message )
    {

		case WM_ACTIVATEAPP:
            ApplicationActive = (wParam == WA_ACTIVE) || (wParam == WA_CLICKACTIVE);
            if ( ApplicationActive )
            {
                // if () to avoid crash on first activate app, when the objects haven't been created, yet
                if ( Cache ) Cache->Invalidate();
                //*LOG << "Cache Invalidated! \n";
            }

        return 0L;


        case WM_DESTROY:
            SAFEDELETE( DXInput );
            SAFEDELETE( Mouse );
            SAFEDELETE( Painter );
            SAFEDELETE( EditFish );
            SAFEDELETE( BackBuffer );
            SAFEDELETE( Primary );
            SAFEDELETE( DirectDraw );
            SAFEDELETE( Font1 );
            SAFEDELETE( Writer );
            SAFEDELETE( Cache );
			SAFEDELETE( KeyBoard );
			SAFEDELETE( MouseCursor );
            ShutDownLogging ();
            PostQuitMessage(0);
        return 0L;

        case WM_SETCURSOR:
            SetCursor( NULL );
        return true;

    }
    return DefWindowProc( WindowHandle, Message, wParam, lParam );
}

/*
 *
 */
void
InitApplication( HINSTANCE hInstance, int nCmdShow )
{

    WNDCLASS    WindowClass;

    // Set up and register window class
    WindowClass.style            = CS_HREDRAW | CS_VREDRAW;
    WindowClass.lpfnWndProc      = WindowProcedure;
    WindowClass.cbClsExtra       = 0;
    WindowClass.cbWndExtra       = 0;
    WindowClass.hInstance        = hInstance;
    WindowClass.hIcon            = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON1));
    WindowClass.hCursor          = LoadCursor(NULL, IDC_ARROW);
    WindowClass.hbrBackground    = (HBRUSH )GetStockObject(BLACK_BRUSH);
    WindowClass.lpszMenuName     = NAME;
    WindowClass.lpszClassName    = NAME;
    RegisterClass( &WindowClass );

	RECT Client;
	SetRect( &Client, 64, 64, 800 + 64, 600 + 64 );
	AdjustWindowRectEx(
		&Client,
		WS_OVERLAPPEDWINDOW,
        false,
		0 );



    // Now that windows knows about our new type of window create one
    WindowHandle = CreateWindowEx(
        0,//WS_EX_TOPMOST,
        NAME,
        TITLE,
        FullScreen ? WS_POPUP : WS_OVERLAPPEDWINDOW,
        Client.left,
        Client.top,
        Client.right - Client.left,
        Client.bottom - Client.top,
        NULL,
        NULL,
        hInstance,
        NULL );

    ASSERT( WindowHandle != NULL );

    ShowWindow( WindowHandle, nCmdShow );
    UpdateWindow( WindowHandle );
    SetFocus( WindowHandle );

    InitLogging ( "EditLog.txt" );

	*LOG << "Init";

    DXInput = new CInput( hInstance );

    Mouse = new CMouse( WindowHandle, DXInput );

    KeyBoard = new CKeyboard( WindowHandle, DXInput );

    DirectDraw = new CDxDraw( WindowHandle, FullScreen );

    Primary = new CSurface();
    Primary->Create( );

    BackBuffer = new CSurface( );
	if ( DirectDraw->IsFullScreen( ))
		BackBuffer->Create( Primary ); // We're fullscreen, create a DX flipping chain
	else
        BackBuffer->Create( 800, 600, true ); // We're windowed, create offcreen vidmem backbuffer

    Painter = new CPaint();
    Painter->SetDrawingTarget( BackBuffer );

    CPixelConverter *PixelConverter = NULL;
    PixelConverter = new CPixelConverter( BackBuffer );
    ASSERT( PixelConverter != NULL );
    tUINT32 TransparentColor = PixelConverter->RGBToColor( 255, 0, 255 );
    SAFEDELETE( PixelConverter );

	// create cache
    Cache = new CCache( 1, 4, 4, 1, TransparentColor );
	ASSERT( Cache != NULL );

    EditFish = new CEditFish( BackBuffer,
							  Cache,
							  "graphics\\BlackFish.ini");

    Font1 = new CTileSet( "graphics\\font.ini", "[Font1]" );
    Font1->SetCache( Cache );

    MouseCursor = new CTileSet( "graphics\\mouse.ini", "[MouseCursor]" );
    MouseCursor->SetCache( Cache );

    Writer = new CWrite();
    Writer->SetFont( Font1 );
    Writer->SetSpacing( 0 );
	*LOG << "Writer initialized" << std::endl;
    //EditFish->SetWriter(Writer);

	srand(static_cast<tUINT32> (Timer.GetTime()));
}

void
Transpatch (CDialog::cRETURNDATABASE* DataPackage)
{
    CNewMapDialog::cRETURNDATA* NewMapData =
        dynamic_cast<CNewMapDialog::cRETURNDATA*> (DataPackage);
	if(NewMapData)
	{
		EditFish->NewMapHeader(DataPackage);
		return;
	}

    CLoadMapDialog::cRETURNDATA* LoadMapData =
        dynamic_cast<CLoadMapDialog::cRETURNDATA*> (DataPackage);
	if(LoadMapData)
	{
		EditFish->LoadMap(DataPackage);
		return;
	}

    CSaveMapDialog::cRETURNDATA* SafeMapData =
        dynamic_cast<CSaveMapDialog::cRETURNDATA*> (DataPackage);
	if(SafeMapData)
	{
		EditFish->SaveMap(DataPackage);
		return;
	}
	// and so on

	//if we get to this point, a wrong or empty DataPackage (Cancel)
	//was returned, destroy dialog anyway
	SAFEDELETE(Dialog);
}

/*
 *
 */
extern "C" int PASCAL
WinMain(
    HINSTANCE Instance,
    HINSTANCE PreviousInstance,
    LPSTR CommandLine,
    int CommandShow)
{
    // !! Dummies to avoid the 'Not used' warning
    PreviousInstance = PreviousInstance;
    CommandLine      = CommandLine;

    InitApplication( Instance, CommandShow );

    MSG Message;
    for (;;)
    {
        if ( PeekMessage( &Message, NULL, 0, 0, PM_NOREMOVE ))
        {
            if ( !GetMessage( &Message, NULL, 0, 0 ))
                return Message.wParam;
            TranslateMessage( &Message );
            DispatchMessage( &Message );
        }
        else if ( ApplicationActive )
        {
			Mouse->UpdateState();
			Mouse->GetState( MouseX, MouseY, WheelPos, Button );

			tUINT32 Click = Mouse->GetLastClick();

			KeyBoard->UpdateState();
			tUINT8 Key = KeyBoard->GetLastKey();
			if (Dialog == NULL)
			{
				EditFish->MouseState(MouseX, MouseY, Button);
				/*
				 * MouseButtonCheck
				 */
                if ( Click & CMouse::Button1 )
				{
					EditFish->MouseClick( MouseX, MouseY, 1 );
				}

                if ( Click & CMouse::Button2 )
				{
					EditFish->MouseClick( MouseX, MouseY, 2 );
				}

                if ( Click & CMouse::Button3 )
				{
					EditFish->MouseClick( MouseX, MouseY, 3 );
				}

				// !! not encapsuled, EditMain should not need to know the button no.
				if ( WheelPos == WHEELUP)
				{
					EditFish->ActivateButton( 0 , 1 );  // same as left click on UpArrow
				}
				// !! dito
				if ( WheelPos == WHEELDOWN)
				{
					EditFish->ActivateButton( 1 , 1 );  // same as left click on DownArrow
				}

				EditFish->Scroll(MouseX, MouseY);

				switch (Key)
				{
					case KEY_ESCAPE:
						PostMessage( WindowHandle, WM_CLOSE, 0, 0 );
					break;

					case KEY_F12:
						PostMessage( WindowHandle, WM_CLOSE, 0, 0 );
					break;

					default :
						EditFish->HandleKeyStroke(Key);
					break;
				}

			}
			else
			{
				// Escape and F12 are the only keys treated normally,
				// but destroy dialog before leaving program
				if (( Key == KEY_ESCAPE ) || ( Key == KEY_F12 ))
				{
					SAFEDELETE (Dialog);
					PostMessage( WindowHandle, WM_CLOSE, 0, 0 );
				}
			    // all other input is treated by the dialog itself
				else
				{
					Dialog->MouseClick(MouseX, MouseY, Click);
                    bool Shift;
					Shift = KeyBoard->GetState(KEY_LSHIFT) | KeyBoard->GetState(KEY_RSHIFT);
					char ASCII = KeyBoard->KeyToASCII( Key, Shift);
					if (ASCII)
					{
						Dialog->CharMessage(ASCII);
					}


                    if ( Dialog->GetState() == CDialog::FINISH )
					{
						Transpatch(Dialog->GetDataPackage());
					}
				}
			}

            Painter->Fill( 0 );
            EditFish->Render(EditFish->GetWallState());
            EditFish->Menu();
            ShowMiscInfo ();

			// check for dialog, if active, blit
			if (Dialog != NULL)
			{
				Dialog->Show(BackBuffer);
			}

			RECT Source, Dest, Screen;
			SetRect( &Screen, 0, 0, 799, 699 );
			Source = MouseCursor->GetTileRect();
			Dest = MouseCursor->GetTileRect();
			Dest.left	+= MouseX;
			Dest.right	+= MouseX;
			Dest.top	+= MouseY;
			Dest.bottom	+= MouseY;
			Clip( Screen, Source, Dest );
			MouseCursor->BlitTrans( 1, &Source, *BackBuffer, &Dest );

			if ( DirectDraw->IsFullScreen( ))
				Primary->Flip( );
			else
			{
				RECT ScreenRect;
				GetClientRect( DirectDraw->GetWindow( ), &ScreenRect);
				ClientToScreen( DirectDraw->GetWindow( ), reinterpret_cast<POINT*>(&ScreenRect.left));
				ClientToScreen( DirectDraw->GetWindow( ), reinterpret_cast<POINT*>(&ScreenRect.right));
				Primary->Blit( *BackBuffer, NULL, &ScreenRect );
			}
        }
        else
        {
            WaitMessage( );
        }
    }
}

