#include "Editfish/Minimap.hpp"

/******************************************************************************
 * BlackFish
 * Copyright (C) 1999-2001 Sren Meyer-Eppler, Jutta Jordans
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/

/*********************************************************************
	Minimap		class that maintains and blits a surface containing
			    information about wall and floor tiles of a map

	History of Changes:
	- 29.09.2000 file created by Capella
**********************************************************************/

#include "Blackfish\Surface.hpp"
#include "Blackfish\Paint.hpp"
#include "Map\Map.hpp"
#include "Blackfish\Pixelconverter.hpp"
//#include "General\Debug.hpp"
//#include "General\Helpmacros.hpp"


cMINIMAP::cMINIMAP()
	: Surface( NULL )
	, MM_SIZE_X( 100 )
	, MM_SIZE_Y( 100 )
{
    Surface = new CSurface();
    Surface->Create(MM_SIZE_X, MM_SIZE_Y, false);
}

cMINIMAP::~cMINIMAP()
{
	SAFEDELETE(Surface);
}

/*
 *	repaints the whole map
 */
void
cMINIMAP::Refresh(CMap* NewMap)
{
	tSINT32 Width, Height;
	if(NewMap->GetWidth() >= MM_SIZE_X)
	{
		Width = MM_SIZE_X;
	}
	else
	{
		Width = NewMap->GetWidth();
	}
	if(NewMap->GetHeight() >= MM_SIZE_X)
	{
		Height = MM_SIZE_X;
	}
	else
	{
		Height = NewMap->GetHeight();
	}

    CPixelConverter Converter(Surface);
    CPaint* Painter = new CPaint();
	Painter->SetDrawingTarget(Surface);
	//Painter->Fill(Converter.RGBToColor(20,20,20));
	Painter->Fill(0, 0, MM_SIZE_X, MM_SIZE_Y, Converter.RGBToColor(0,0,0));
	Painter->Fill( 0, 0, Width, Height, Converter.RGBToColor(30,30,30));
	Painter->BeginDrawing();

	for (tSINT32 y = 0; y < Height; y++)
	{
		for (tSINT32 x = 0; x < Width; x++)
		{
			if(NewMap->GetWall( x, y ) != NULL)
			{
				Painter->PutPixel(x, y, Converter.RGBToColor(0,192,192));
			}
			else if (( NewMap->GetFirstLayer(x,y)->TileNr != 0 ) ||
					 ( NewMap->GetFirstLayer(x,y)->TileSet != 0 ))
			{
				Painter->PutPixel(x, y, Converter.RGBToColor(64,64,64));
			}
		}
	}
*LOG << "Complete MiniMap refreshed!" << std::endl;
	Painter->EndDrawing();
	SAFEDELETE( Painter );
}

void
cMINIMAP::Blit(CSurface* Dest, tSINT32 XKoord, tSINT32 YKoord )
{
	RECT SourceRect;
	SourceRect.left   = 0;
	SourceRect.right  = MM_SIZE_X;
	SourceRect.top    = 0;
	SourceRect.bottom = MM_SIZE_Y;
	RECT DestRect;
	DestRect.left	= XKoord;
	DestRect.right	= DestRect.left + MM_SIZE_X;
	DestRect.top	= YKoord;
	DestRect.bottom	= DestRect.top + MM_SIZE_Y;

	Dest->Blit(*Surface, &SourceRect, &DestRect);
}

void
cMINIMAP::RefreshPixel(CMap* Map, tSINT32 PosX, tSINT32 PosY)
{
	if(( PosX < MM_SIZE_X ) && ( PosY < MM_SIZE_Y ))
	{
        CPixelConverter Converter(Surface);
        CPaint* Painter = new CPaint();
		Painter->SetDrawingTarget(Surface);
		Painter->BeginDrawing();
		if(Map->GetWall( PosX, PosY ) != NULL)
		{
			Painter->PutPixel(PosX, PosY, Converter.RGBToColor(0,192,192));
		}
		else if (( Map->GetFirstLayer(PosX, PosY)->TileNr != 0 ) ||
				 ( Map->GetFirstLayer(PosX, PosY)->TileSet != 0 ))
		{
			Painter->PutPixel(PosX, PosY, Converter.RGBToColor(64,64,64));
		}
		else
		{
			Painter->PutPixel(PosX, PosY, Converter.RGBToColor(20,20,20));
		}
		Painter->EndDrawing();
		SAFEDELETE(Painter);
	}
}
