#include "Selection.hpp"
/*********************************************************************
	HawkCreekSoftware

	Selection, contains a class to maintain a selection of integers,
	used for the random function of the Editor, is basically a set of
	integers

	History of Changes:

	- 25.09.2000	file created by Capella
**********************************************************************/

#include <stdlib.h>
#include "general\helpmacros.hpp"
//#include "general\debug.hpp"


cSELECTION::cSELECTION()
	: NrOfSelectedTiles( 0 )
	, SelectArray( NULL )
{
}

cSELECTION::~cSELECTION()
{
	ReleaseSelection();
}

/*
 *	add a value to the selection by making a complete new array
 *  will be rather slow for big selections, since all values have
 *  to be moved, but our selections will be rather small
 */
void
cSELECTION::Select(tSINT32 Value)
{
	if(IsSelected(Value))
		return;
	else
	{
		// copy old array and add new value
		tUINT32 NewNrOfTiles = NrOfSelectedTiles + 1;
		tSINT32* NewArray = new tSINT32[NewNrOfTiles];
		ASSERT( NewArray != NULL );

		for (tUINT32 i=0; i < NrOfSelectedTiles; i++)
		{
			ASSERT( SelectArray != NULL );
			NewArray[i] = SelectArray[i];
		}
		NewArray[NewNrOfTiles - 1] = Value;

		// delete the old array and bend pointer
		ReleaseSelection();
		SelectArray = NewArray;
		NrOfSelectedTiles = NewNrOfTiles;
	}
}

/*
 *	remove a value from the selection, works like select
 */
void
cSELECTION::Deselect(tSINT32 Value)
{

	if(!IsSelected(Value))
		return;
	else
	{
		tUINT32 NewNrOfTiles = NrOfSelectedTiles - 1;
		tSINT32* NewArray = new tSINT32[NewNrOfTiles];
		ASSERT( NewArray != NULL );
		// copy all but the element containing the value to the new array
		tUINT32 Index = 0;
		for (tUINT32 i = 0; i < NrOfSelectedTiles; i++)
		{
			ASSERT( SelectArray != NULL );
			if (SelectArray[i] != Value)
			{
				NewArray[Index] = SelectArray[i];
				*LOG << "OldArray[" << Index << "] = " << SelectArray[Index] << std::endl;
				Index++;
			}

		}

		//delete the old array and bend the pointer to the new one
		ReleaseSelection();
		SelectArray = NewArray;
		NrOfSelectedTiles = NewNrOfTiles;
	}
}


/*
 *	check if a value is part of the selection
 */
bool
cSELECTION::IsSelected(tSINT32 Value)
{

	for (tUINT32 i = 0; i < NrOfSelectedTiles; i++)
	{
		ASSERT( SelectArray != NULL );
		if(SelectArray[i] == Value)
            return true;
	}
    return false;
}


/*
 *	give a random value from the selection
 */
tSINT32
cSELECTION::GetRandomSelection()
{
	if(NrOfSelectedTiles == 0)
		return -1;

	ASSERT( SelectArray != 0 );
	// get a random number between 0 and NrOfSelectedTiles-1 (array index)
	tSINT32 Random = static_cast<tSINT32>
		(((static_cast<double> (rand()) / static_cast<double> (RAND_MAX + 1)) * NrOfSelectedTiles));

	return SelectArray[Random];
}

/*
 *	delete the entire Selection
 */
void
cSELECTION::ReleaseSelection()
{
	SAFEDELETEARRAY( SelectArray );
	NrOfSelectedTiles = 0;
}
