#include "debug.hpp"

/******************************************************************************
 * BlackFish
 * Copyright (C) 1999-2001 Sren Meyer-Eppler, Jutta Jordans
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/


#pragma warning ( push, 1 )
#include <fstream>
#include <cstdlib>                     // exit()
#pragma warning ( pop )

#include "General\HelpMacros.hpp"


std::ofstream *LOG;

const char DebugFileName[80] = "assert.txt";
const unsigned int MaxAssertionFailures = 1;

/*
 * write an assertion failure to a text file
 */
void
FileAssert( bool Expression, const char *ErrorFileName, unsigned int ErrorLineNr )
{
    // die wievielte ASSERTION schlgt gerade fehl?
    static unsigned int Counter = 0;
    static std::ofstream OutPutFile( DebugFileName, std::ios_base::out | std::ios_base::trunc );

    // Ist die zu berprfende Bedingung falsch?
    // ja: ffne Datei und schreib wo es schiefgegangen ist
    // nein: do nothing
    if ( !Expression )
    {
        OutPutFile << "(";
        OutPutFile.width( 5 );
        OutPutFile << Counter << ")" << " An assertion failed in file '" \
			<< ErrorFileName << "', line " << ErrorLineNr << std::endl;
        Counter++;
        if ( Counter >= MaxAssertionFailures )
			QUIT;
    }
    // !!! I'm relying on windows here to close the outputfile
}

/*
 * prepare the LOG file output stream, to be called once at program start
 */
void
InitLogging( const char *LogFileName )
{
    LOG = new std::ofstream( LogFileName, std::ios_base::out | std::ios_base::trunc );
//	LOG->setbuf( NULL, 0 );		// !! using C++ standard streams now... find out how to write unbuffered
}

/*
 * delete the LOG file output stream, to be called once at program termination
 */
void
ShutDownLogging( void )
{
    SAFEDELETE( LOG );
}
