#ifndef __TYPE_HPP__
#define __TYPE_HPP__

/******************************************************************************
 * BlackFish
 * Copyright (C) 1999-2001 Sren Meyer-Eppler, Jutta Jordans
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/

//-------------------------------------------------------------------------------------------------
///    Purpose:
///    Header file to declare commonly used data types like integer and floating point
///    numbers. Used so that names are consistent and shorter, i.e. tUINT32 instead of
///    unsigned int. Also useful for porting the system to another compiler/processor
///    which might use different names/sizes.
///    This header should be included by every file in the project and the declared
///    data types must always be used instead of the compiler/language default.
//-------------------------------------------------------------------------------------------------

#include <cstddef>		// NULL

/**
 * Names are all capital letters and consist of:
 * prefix: U for unsigned, S for signed
 * name: INT for integer
 * suffix: 8,16,32 to represent the size of this data type in bits
 */
typedef unsigned char           tUINT8;  //           0..255
typedef signed char             tSINT8;  //        -128..127
typedef unsigned short int      tUINT16; //           0..65535
typedef signed short int        tSINT16; //      -32768..32767
typedef unsigned long int       tUINT32; //           0..4294967295
typedef signed long int         tSINT32; // -2147483648..2147483647
typedef unsigned __int64		tUINT64; //			  huge
typedef signed __int64			tSINT64; //			  huge

/**
 * Special, non-integer types:
 * floating point numbers, characters, BOOLEAN
 */
/// tFLOAT, smallest absolute value 1.1E-38, largest absolute value 3.4E+38, digits of accuracy 6
typedef float                   tFLOAT;

/// tDOUBLE, smallest absoulte value 2.2E-308, largest absolute value 1.7E+308, digits of accuracy 15
typedef double                  tDOUBLE;

#endif
