#include "Vector.hpp"

/******************************************************************************
 * BlackFish
 * Copyright (C) 1999-2001 Sren Meyer-Eppler, Jutta Jordans
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/


#pragma warning ( push, 1 )
#include <math.h>
#pragma warning ( pop )

#include "General/Debug.hpp"

/*
 *
 */
CMatrix3x3::CMatrix3x3( void )
{
	Set( 1, 0, 0,
		 0, 1, 0,
		 0, 0, 1 );
}

/*
 *
 */
CMatrix3x3::CMatrix3x3( tFLOAT x1, tFLOAT y1, tFLOAT z1,
						tFLOAT x2, tFLOAT y2, tFLOAT z2,
						tFLOAT x3, tFLOAT y3, tFLOAT z3 )
{
	Set( x1, y1, z1,
		 x2, y2, z2,
		 x3, y3, z3 );
}

/*
 *
 */
void
CMatrix3x3::Set( tFLOAT x1, tFLOAT y1, tFLOAT z1,
				 tFLOAT x2, tFLOAT y2, tFLOAT z2,
				 tFLOAT x3, tFLOAT y3, tFLOAT z3 )
{
	this->x1 = x1; this->y1 = y1; this->z1 = z1;
	this->x2 = x2; this->y2 = y2; this->z2 = z2;
	this->x3 = x3; this->y3 = y3; this->z3 = z3;
}

/*
 *
 */
CVector::CVector( void )
{
	Set( 0, 0, 0 );
}

/*
 *
 */
CVector::CVector( tFLOAT x, tFLOAT y, tFLOAT z )
{
	Set( x, y, z );
}

/*
 *
 */
void
CVector::Set( tFLOAT x, tFLOAT y, tFLOAT z )
{
	this->x = x;
	this->y = y;
	this->z = z;
}

/*
 *
 */
void
CVector::Set( CVector Vector )
{
	this->x = Vector.x;
	this->y = Vector.y;
	this->z = Vector.z;
}

/*
 *
 */
void
CVector::Multiply( CMatrix3x3 *Matrix, CVector *Vector )
{
	x = Vector->x * Matrix->x1 + Vector->y * Matrix->x2 + Vector->z * Matrix->x3;
	y = Vector->x * Matrix->y1 + Vector->y * Matrix->y2 + Vector->z * Matrix->y3;
	z = Vector->x * Matrix->z1 + Vector->y * Matrix->z2 + Vector->z * Matrix->z3;
}

/*
 *
 */
void
CVector::Multiply( tFLOAT Skalar )
{
	x *= Skalar;
	y *= Skalar;
	z *= Skalar;
}

/*
 *
 */
void
CVector::Add( CVector Vector )
{
	x += Vector.x;
	y += Vector.y;
	z += Vector.z;
}

/*
 *
 */
void
CVector::Subtract( CVector Vector )
{
	x -= Vector.x;
	y -= Vector.y;
	z -= Vector.z;
}

/*
 *
 */
tFLOAT
CVector::GetLength( void )
{
	return ((tFLOAT) sqrt( x*x + y*y + z*z ));
}

/*
 *
 */
void
CVector::Normalize( void )
{
	tFLOAT Length = GetLength( );
	if ( Length )
	{
		x /= Length;
		y /= Length;
		z /= Length;
	}
}

/*
 *
 */
bool
CVector::Equals( CVector Vector )
{
	const tFLOAT TOLERANCE = 0.02f; // float values can't be compared exactly

    bool ReturnValue = false;
	ReturnValue = ( fabs( Vector.x - x ) < TOLERANCE );
	ReturnValue = ( fabs( Vector.y - y ) < TOLERANCE ) && ReturnValue;
	ReturnValue = ( fabs( Vector.z - z ) < TOLERANCE ) && ReturnValue;
	return ReturnValue;
}
