#ifndef __MAP_HPP__
#define __MAP_HPP__

/******************************************************************************
 * BlackFish
 * Copyright (C) 1999-2001 Sren Meyer-Eppler, Jutta Jordans
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************************/

/******************************************************************************
	Header for MapClass

	19.03.2000: file generated by Capella
	07.10.2000: BuschnicK
				this file is a mess, too many confusing accessor functions, each
				meaning a sliglty different thing... difficult to clean up though,
				since we already use this everywhere
*******************************************************************************/

#pragma warning ( push, 1 )
#include <list>
#include <vector>
#include <string>
#pragma warning ( pop )

#ifndef __TYPE_HPP__
#include "general\type.hpp"
#endif

#ifndef __HELPMACROS_HPP__
#include "general\helpmacros.hpp"
#endif

class CSprite; // must not be included here, since Sprite.hpp includes Map.hpp
class CVector;

typedef enum
{
	NONE		= 0,
	NORTHWEST,
	NORTH,
	NORTHEAST,
	EAST,
	SOUTHEAST,
	SOUTH,
	SOUTHWEST,
	WEST
} tDIRECTION;

/******************************************************************************************/

class // used like a struct, i.e. data members are modified from outside
CWallinfo
{
public:
	tSINT32 TileSet;
	tSINT32 TileNr;

    CWallinfo( tSINT32 TileSet = 0, tSINT32 TileNr = 0 )
		: TileSet( TileSet ), TileNr( TileNr ) { };
};

/******************************************************************************************/

class // used like a struct, i.e. data members are modified from outside
CCellinfo
{
public:
    CCellinfo *NextLayer;
	tSINT32 TileSet;
	tSINT32 TileNr;

    CCellinfo( tSINT32 TileSet = 0, tSINT32 TileNr = 0 )
		: NextLayer( NULL ), TileSet( TileSet ), TileNr( TileNr ) { };
};

/******************************************************************************************/

class
CBaseCell
{
public:
    CBaseCell( void );
    ~CBaseCell( );

    inline CWallinfo *GetWall( void );
	inline void SetWall( tSINT32 TileSet, tSINT32 TileNr );
	inline void DeleteWall( );

    inline CCellinfo *GetFirstLayer( void );
    inline void SetFirstLayer( tSINT32 TileSet, tSINT32 TileNr, CCellinfo *NextLayer );
	inline void DeleteAllLayers( void );
	inline void AddTopLayer( tSINT32 TileSet, tSINT32 TileNr );
	inline void RemoveTopLayer( void );

    bool IsBlocking( void );

    inline std::list<CSprite *> *GetSpriteList( void ) { return &SpriteList; };

	inline tUINT32 GetFlags( void )			{ return Flags; };
	inline void SetFlags( tUINT32 Flags ) 	{ this->Flags = Flags; };

private:
    CCellinfo              FirstLayer;
    CWallinfo             *Wall;
	tUINT32				  Flags;
    std::list<CSprite *>  SpriteList;

	// copy and assignment is not allowed (although it probably should be...)
    CBaseCell( const CBaseCell &BaseCell );
    CBaseCell & operator = ( const CBaseCell &BaseCell );
};

/******************************************************************************************/
// !! this should be a private subclass of CMap, since noone else has a use for CNodes
//    not yet implemented that way because the operator > and greater functions will get
//    a little tricky
class
CNode
{
public:
	tSINT32 x, y;
	tSINT32 Cost;
	tSINT32 Heuristic;
	tSINT32 Total;
    CNode *Parent;

    CNode( tSINT32 x, tSINT32 y, tSINT32 Cost, tSINT32 Heuristic, CNode *Parent = NULL )
		: x(x), y(y), Cost(Cost), Heuristic(Heuristic), Parent(Parent), Total( Cost + Heuristic )
	{
		ASSERT( Total >= 0 ); // an overflow has occured
	}

    friend bool operator > ( const CNode &a, const CNode &b );
};

/******************************************************************************************/

class
CMap
{
public:

	enum {	DATAVERSION = 3 };
	enum {	FLAGBLOCKING	= 2 << 0,
			FLAGTRANSPARENT = 2 << 1 };

	struct
	tMAPHEADER
	{
	public:
		tSINT32 Version;		// 4
		char	Name[64];		// 64
		char	Author[64];		// 64
		tSINT32	Width;			// 4
		tSINT32 Height;			// 4
		tSINT8	Padding[126];	// 126
	};							// 256 total

    CMap( );
    CMap( const char *NewFileName );
    CMap( const tMAPHEADER & MapHeader );
    ~CMap( );

    inline CCellinfo *GetFirstLayer( tSINT32 PosX, tSINT32 PosY );
    inline void SetFirstLayer( tSINT32 PosX, tSINT32 PosY, tSINT32 TileSet, tSINT32 TileNr, CCellinfo *NextLayer );
	inline void DeleteAllLayers( tSINT32 PosX, tSINT32 PosY );
	inline void AddTopLayer( tSINT32 PosX, tSINT32 PosY, tSINT32 TileSet, tSINT32 TileNr );
	inline void RemoveTopLayer( tSINT32 PosX, tSINT32 PosY );

    inline CWallinfo *GetWall( tSINT32 PosX, tSINT32 PosY );
	inline void SetWall( tSINT32 PosX, tSINT32 PosY, tSINT32 TileSet, tSINT32 TileNr );
	inline void DeleteWall( tSINT32 PosX, tSINT32 PosY );

	inline tUINT32 GetFlags( tSINT32 PosX, tSINT32 PosY );
	inline void SetFlags( tSINT32 PosX, tSINT32 PosY, tUINT32 Flags );

    inline bool IsBlocking( tSINT32 StartX, tSINT32 StartY, tSINT32 DestX, tSINT32 DestY );
    inline bool IsBlocking( tSINT32 StartX, tSINT32 StartY );
    inline bool IsValidCell( tSINT32 x, tSINT32 );

    inline std::list<CSprite *> *GetSpriteList( tSINT32 PosX, tSINT32 PosY );

	inline std::string GetFileName()	{ return FileName; };
	inline tSINT32 GetWidth( void )		{ return Header.Width; };
	inline tSINT32 GetHeight( void )	{ return Header.Height; };

	void Optimize( void );

	void Load( const char *FileName );
	void Save( const char *FileName );
	void SetName( const char* MapName );
	void SetAuthor( const char* Author );
	const char* GetName();
	const char* GetAuthor();

    void InsertSprite( CVector *Position, CSprite *Sprite );
    void RemoveSprite( CVector *Position, CSprite *Sprite );

    bool FindPath( const CVector & PointA, const CVector & PointB, std::list<CVector> *Path );

private:

	tMAPHEADER Header;
    CBaseCell* Cell;
	std::string FileName;

	void DeleteCells( void );

    inline CBaseCell *GetBaseCell( tSINT32 PosX, tSINT32 PosY );

	enum { COST_ALIGNED = 10, COST_DIAGONAL = 14 };
	enum { MAXITERATIONS = 1000 };	// how long should I try until giving up? (there is no path to goal or path too long)
	tSINT32 GetHeuristic( tSINT32 x1, tSINT32 y1, tSINT32 x2, tSINT32 y2 );
	tSINT32 GetCost( tSINT32 x1, tSINT32 y1, tSINT32 x2, tSINT32 y2 );
    void AddNode( std::vector< CNode * > &Open, std::vector< CNode * > &Closed, CNode *Node );
    void DeleteNodes( std::vector< CNode * > &Open, std::vector< CNode * > &Closed );

	// copy and assignment is not allowed
    CMap( const CMap &Map );
    CMap & operator = ( const CMap &Map );
};

/******************************************************************************************/

/*
 *
 */
bool
CMap::IsBlocking( tSINT32 StartX, tSINT32 StartY, tSINT32 DestX, tSINT32 DestY )
{
	ASSERT( abs( DestX - StartX ) <= 1 );
	ASSERT( abs( DestY - StartY ) <= 1 );

	// if one of the cells is invalid (out of map bounds) we have to exit early or we'll
	// get access violations for IsBlocking( ) checks
	if ( !IsValidCell( DestX, DestY ) || !IsValidCell( StartX, StartY ))
        return true;

    bool Blocks = GetBaseCell( DestX, DestY )->IsBlocking( );

	// if we are moving N, S, E, or W we are moving diagonal in arrayspace and all
	// four touching cells must be passable, i.e. do not cut corners
	// note that these accesses cannot be out of bounds because our maps are convex (rectangle)
	// [TODO] SME, allow certain types of objects to be cut diagonally, i.e. I don't want
	//        walls to be passable diagonally but for example marines, since they do not
	//        occupy a whole square, at least visually
	//        Note that this would require Cell->IsBlocking() to differentiate between
	//        blocking sprites and blocking walls
	tSINT32 dx = DestX - StartX;
	tSINT32 dy = DestY - StartY;
	if (( dx != 0 ) && ( dy != 0 ))
		Blocks |= GetBaseCell( DestX - dx, DestY )->IsBlocking( )
			   || GetBaseCell( DestX, DestY - dy )->IsBlocking( );

	return Blocks;
}

/*
 *
 */
bool
CMap::IsBlocking( tSINT32 StartX, tSINT32 StartY )
{
	return IsBlocking( StartX, StartY, StartX, StartY );
}

/*
 *
 */
bool
CMap::IsValidCell( tSINT32 x, tSINT32 y )
{
	return ((x >= 0) && (x < GetWidth( )) && (y >= 0) && (y < GetHeight( )));
}

/*
 *
 */
CCellinfo *
CMap::GetFirstLayer( tSINT32 PosX, tSINT32 PosY )
{
	return GetBaseCell( PosX, PosY )->GetFirstLayer( );
}

/*
 *
 */
void
CMap::SetFirstLayer( tSINT32 PosX, tSINT32 PosY, tSINT32 TileSet, tSINT32 TileNr, CCellinfo *NextLayer )
{
	GetBaseCell( PosX, PosY )->SetFirstLayer( TileSet, TileNr, NextLayer );
}

/*
 *
 */
void
CMap::DeleteAllLayers( tSINT32 PosX, tSINT32 PosY )
{
	GetBaseCell( PosX, PosY )->DeleteAllLayers( );
}

/*
 *
 */
void
CMap::AddTopLayer( tSINT32 PosX, tSINT32 PosY, tSINT32 TileSet, tSINT32 TileNr )
{
	GetBaseCell( PosX, PosY )->AddTopLayer( TileSet, TileNr );
}

/*
 *
 */
void
CMap::RemoveTopLayer( tSINT32 PosX, tSINT32 PosY )
{
	GetBaseCell( PosX, PosY )->RemoveTopLayer( );
}

/*
 *
 */
CWallinfo *
CMap::GetWall( tSINT32 PosX, tSINT32 PosY )
{
	return GetBaseCell( PosX, PosY )->GetWall( );
}

/*
 *
 */
void
CMap::SetWall( tSINT32 PosX, tSINT32 PosY, tSINT32 TileSet, tSINT32 TileNr )
{
	GetBaseCell( PosX, PosY )->SetWall( TileSet, TileNr );
}

/*
 *
 */
void
CMap::DeleteWall( tSINT32 PosX, tSINT32 PosY )
{
	GetBaseCell( PosX, PosY )->DeleteWall( );
}

/*
 *
 */
CBaseCell *
CMap::GetBaseCell( tSINT32 PosX, tSINT32 PosY )
{
	return &Cell[PosY * Header.Width + PosX];
}

/*
 *
 */
tUINT32
CMap::GetFlags( tSINT32 PosX, tSINT32 PosY )
{
	return GetBaseCell( PosX, PosY )->GetFlags( );
}

/*
 *
 */
void
CMap::SetFlags( tSINT32 PosX, tSINT32 PosY, tUINT32 Flags )
{
	GetBaseCell( PosX, PosY )->SetFlags( Flags );
}

std::list<CSprite *> *
CMap::GetSpriteList( tSINT32 PosX, tSINT32 PosY )
{
	return GetBaseCell( PosX, PosY )->GetSpriteList( );
}



/******************************************************************************************/

/*
 *
 */
CWallinfo *
CBaseCell::GetWall( void )
{
	return Wall;
}

/*
 *
 */
void
CBaseCell::SetWall( tSINT32 TileSet, tSINT32 TileNr )
{
	DeleteWall( ); // just in case there already is one
    Wall = new CWallinfo( TileSet, TileNr );
	ASSERT( Wall != NULL );
    Flags |= CMap::FLAGBLOCKING;
}

/*
 *
 */
void
CBaseCell::DeleteWall( )
{
	SAFEDELETE( Wall );
    Flags = Flags & ~CMap::FLAGBLOCKING;
}

/*
 *
 */
CCellinfo *
CBaseCell::GetFirstLayer( void )
{
	return &FirstLayer;
}

/*
 *
 */
void
CBaseCell::SetFirstLayer( tSINT32 TileSet, tSINT32 TileNr, CCellinfo *NextLayer )
{
	FirstLayer.TileSet		= TileSet;
	FirstLayer.TileNr		= TileNr;
	FirstLayer.NextLayer	= NextLayer;
}

/*
 *
 */
void
CBaseCell::DeleteAllLayers( )
{
	while ( FirstLayer.NextLayer != NULL )
	{
        CCellinfo *Layer = FirstLayer.NextLayer;
        CCellinfo *OldLayer = &FirstLayer;

		while ( Layer->NextLayer != NULL )
		{
			OldLayer	= Layer;
			Layer		= Layer->NextLayer;
		}

		SAFEDELETE( Layer );

		Layer				= NULL;
		OldLayer->NextLayer = NULL;
	}
}

/*
 *
 */
void
CBaseCell::AddTopLayer( tSINT32 TileSet, tSINT32 TileNr )
{
    CCellinfo *Iterator = &FirstLayer;
	while ( Iterator->NextLayer )
		Iterator = Iterator->NextLayer;

    Iterator->NextLayer = new CCellinfo( TileSet, TileNr );
}

/*
 *
 */
void
CBaseCell::RemoveTopLayer( )
{
    CCellinfo *Iterator = &FirstLayer;
	while (( Iterator->NextLayer ) && ( Iterator->NextLayer->NextLayer ))
		Iterator = Iterator->NextLayer;

	SAFEDELETE( Iterator->NextLayer );

	Iterator->NextLayer = NULL;
}

#endif
